# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
"""Assets module"""

import superdesk
import bson.errors

from werkzeug.wsgi import wrap_file
from flask import request, current_app as app
from content_api.errors import FileNotFoundError
from superdesk import get_resource_service
from superdesk.upload import upload_url as _upload_url

bp = superdesk.Blueprint('assets', __name__)
cache_for = 3600 * 24 * 7  # 7 days cache


@bp.route('/assets/<path:media_id>', methods=['GET'])
def get_media_streamed(media_id):
    if not app.auth.authorized([], 'assets', 'GET'):
        return app.auth.authenticate()
    try:
        media_file = app.media.get(media_id, 'upload')
    except bson.errors.InvalidId:
        media_file = None
    if media_file:
        data = wrap_file(request.environ, media_file, buffer_size=1024 * 256)
        response = app.response_class(
            data,
            mimetype=media_file.content_type,
            direct_passthrough=True)
        response.content_length = media_file.length
        response.last_modified = media_file.upload_date
        response.set_etag(media_file.md5)
        response.cache_control.max_age = cache_for
        response.cache_control.s_max_age = cache_for
        response.cache_control.public = True
        response.make_conditional(request)
        response.headers['Content-Disposition'] = 'inline'
        get_resource_service('api_audit').audit_item({'type': 'asset', 'uri': request.url}, media_id)
        return response
    raise FileNotFoundError('File not found on media storage.')


def upload_url(media_id):
    return _upload_url(media_id, view='assets.get_media_streamed')


def init_app(app):
    superdesk.blueprint(bp, app)
    app.upload_url = upload_url



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.io.registry import register_feeding_service
from superdesk.io.feeding_services import FeedingService
from superdesk.errors import IngestApiError
import requests
import traceback
# from pytz import timezone


WUFOO_URL = 'https://{subdomain}.wufoo.com/api/v3/'
WUFOO_QUERY_FORM = 'forms/{form_hash}/'
WUFOO_QUERY_FIELDS = 'fields.json'
WUFOO_QUERY_ENTRIES = 'entries.json'


class WufooFeedingService(FeedingService):
    """
    Feeding Service class which can read article(s) using Wufoo API
    """

    NAME = 'wufoo'

    ERRORS = [IngestApiError.apiTimeoutError().get_error_description(),
              IngestApiError.apiRedirectError().get_error_description(),
              IngestApiError.apiRequestError().get_error_description(),
              IngestApiError.apiGeneralError().get_error_description()]

    label = 'Wufoo feed API'

    fields = [
        {
            'id': 'wufoo_username', 'type': 'text', 'label': 'Login',
            'placeholder': 'Wufoo login', 'required': True
        },
        {
            'id': 'wufoo_api_key', 'type': 'password', 'label': 'API key',
            'placeholder': 'Wufoo API Key', 'required': True
        }
    ]

    parser_restricted_values = ['wufoo']

    def __init__(self):
        self.fields_cache = {}

    def _update(self, provider, update):
        user = provider['config']['wufoo_username']
        wufoo_data = {
            "url": WUFOO_URL.format(subdomain=user),
            "user": user,
            "api_key": provider['config']['wufoo_api_key'],
            "form_query_entries_tpl": WUFOO_QUERY_FORM + WUFOO_QUERY_ENTRIES,
            "update": update}
        try:
            parser = self.get_feed_parser(provider, None)
        except requests.exceptions.Timeout as ex:
            raise IngestApiError.apiTimeoutError(ex, provider)
        except requests.exceptions.TooManyRedirects as ex:
            raise IngestApiError.apiRedirectError(ex, provider)
        except requests.exceptions.RequestException as ex:
            raise IngestApiError.apiRequestError(ex, provider)
        except Exception as error:
            traceback.print_exc()
            raise IngestApiError.apiGeneralError(error, self.provider)
        items = parser.parse(wufoo_data, provider)
        return [items]


register_feeding_service(WufooFeedingService)



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : mugur
# Creation: 2017-11-24 19:54

from copy import deepcopy
from superdesk.commands.data_updates import DataUpdate

from eve.utils import config


class DataUpdate(DataUpdate):
    replace_values_forward = {
        'picture': 'media',
        'unorderedlist': 'unordered list',
        'orderedlist': 'ordered list',
        'anchor': 'link',
        'removeFormat': None
    }
    replace_values_backward = {
        'media': 'picture',
        'unordered list': 'unorderedlist',
        'ordered list': 'orderedlist',
        'link': 'anchor'
    }

    resource = 'content_types'

    def forwards(self, mongodb_collection, mongodb_database):
        self._process_content_type(mongodb_collection, self.replace_values_forward)

    def backwards(self, mongodb_collection, mongodb_database):
        self._process_content_type(self.replace_values_backward)

    def _process_content_type(self, mongodb_collection, replace_values):
        for content_type in mongodb_collection.find({}):
            if 'editor' not in content_type:
                continue
            original_editor = deepcopy(content_type['editor'])
            for field, description in content_type['editor'].items():
                if description and description.get('formatOptions'):
                    for original, new in replace_values.items():
                        if original in description['formatOptions']:
                            description['formatOptions'].remove(original)
                            if new:
                                description['formatOptions'].append(new)
            if original_editor != content_type['editor']:
                print('update editor in content type', content_type['label'])
                mongodb_collection.update({'_id': content_type.get(config.ID_FIELD)},
                                          {'$set': {
                                              'editor': content_type['editor']
                                          }})



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import flask
from flask import request, current_app as app
from superdesk import utils as utils, get_resource_service
from superdesk.services import BaseService
from eve.utils import config
from superdesk.errors import SuperdeskApiError
from apps.auth.errors import UserDisabledError
from superdesk.users.errors import UserInactiveError


class AuthService(BaseService):

    def authenticate(self, document):
        """Authenticate user according to credentials

        :param documents: credentials for this authentication mechanism
        :return: authenticated user
        :raise: CredentialsAuthError if authentication is invalid
        """
        raise NotImplementedError()

    def on_create(self, docs):
        for doc in docs:
            user = self.authenticate(doc)
            if not user:
                raise ValueError()
            if 'is_enabled' in user and not user.get('is_enabled', False):
                raise UserDisabledError()
            if not user.get('is_active', False):
                raise UserInactiveError()
            self.set_auth_default(doc, user['_id'])

    def on_created(self, docs):
        for doc in docs:
            get_resource_service('preferences').set_session_based_prefs(doc['_id'], doc['user'])

    def on_deleted(self, doc):
        """Runs on delete of a session

        :param doc: A deleted auth doc AKA a session
        :return:
        """
        # notify that the session has ended
        app.on_session_end(doc['user'], doc['_id'])

    def set_auth_default(self, doc, user_id):
        doc['user'] = user_id
        doc['token'] = utils.get_random_string(40)
        doc.pop('password', None)

    def update_session(self, updates=None):
        """Update current session with given data.

        :param updates: updates to be made
        """
        if not updates:
            updates = {}
        self.system_update(flask.g.auth['_id'], updates, flask.g.auth)


class UserSessionClearService(BaseService):

    def delete(self, lookup):
        """Delete user session.

        Deletes all the records from auth and corresponding
        session_preferences from user collections
        If there are any orphan session_preferences exist they get deleted as well
        """
        users_service = get_resource_service('users')
        user_id = request.view_args['user_id']
        user = users_service.find_one(req=None, _id=user_id)
        sessions = get_resource_service('auth').get(req=None, lookup={'user': user_id})

        error_message = self.__can_clear_sessions(user)
        if error_message:
            raise SuperdeskApiError.forbiddenError(message=error_message)

        # Delete all the sessions
        for session in sessions:
            get_resource_service('auth').delete_action({config.ID_FIELD: str(session[config.ID_FIELD])})

        # Check if any orphan session_preferences exist for the user
        if user.get('session_preferences'):
            # Delete the orphan sessions
            users_service.patch(user[config.ID_FIELD], {'session_preferences': {}})

        return [{'complete': True}]

    def __can_clear_sessions(self, user):
        """Checks if the session clear request is Invalid.

        Operation is invalid if one of the below is True:
            1. Check if the user exists.
            2. Check if the user is clearing his/her own sessions.

        :return: error message if invalid.
        """

        if not user:
            return 'Invalid user to clear sessions.'

        if str(user[config.ID_FIELD]) == str(flask.g.user[config.ID_FIELD]):
            return 'Not allowed to clear your own sessions.'



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from .data_layer import DataLayer
from eve.utils import ParsedRequest, config, document_etag
from eve import ID_FIELD


class BaseProxy(DataLayer):
    """Data layer implementation used to connect the models to the data layer.

    Transforms the model data layer API into Eve data layer calls.
    """

    def __init__(self, data_layer):
        self.data_layer = data_layer

    def etag(self, doc):
        return doc.get(config.ETAG, document_etag(doc))

    def find_one(self, resource, filter, projection):
        req = ParsedRequest()
        req.args = {}
        req.projection = projection
        return self.data_layer.find_one(resource, req, **filter)

    def find(self, resource, lookup, projection, **options):
        req = ParsedRequest()
        req.args = {}
        req.projection = projection
        return self.data_layer.get(resource, req, lookup)

    def create(self, resource, docs):
        return self.data_layer.create(resource, docs)

    def update(self, resource, filter, doc):
        return self._update(resource, filter, doc)

    def replace(self, resource, filter, doc):
        return self._update(resource, filter, doc, method='replace')

    def delete(self, resource, filter):
        return self.data_layer.delete(resource, filter)

    def _update(self, resource, filter, doc, method='update'):
        _id = doc.pop(ID_FIELD, None)
        original = self.find_one(resource, filter, None)
        filter[ID_FIELD] = original[ID_FIELD]  # make sure it's correct type
        updates = doc.copy()
        res = getattr(self.data_layer, method)(resource, filter[ID_FIELD], updates, original)
        doc.setdefault(ID_FIELD, _id)
        return res



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk

from copy import deepcopy
from datetime import datetime
from flask import current_app as app
from eve.io.mongo import Validator
from superdesk.metadata.item import ITEM_TYPE
from superdesk.logging import logger
from superdesk.text_utils import get_text
from superdesk import get_resource_service
from _collections_abc import MutableMapping

REQUIRED_FIELD = 'is a required field'
STRING_FIELD = 'require a string value'
DATE_FIELD = 'require a date value'
REQUIRED_ERROR = '{} is a required field'


def check_json(doc, field, value):
    if isinstance(doc, dict):
        if field in doc and doc[field] == value:
            return True
        for key in doc:
            if check_json(doc[key], field, value):
                return True
        return False
    elif isinstance(doc, list):
        for item in doc:
            if check_json(item, field, value):
                return True
        return False
    else:
        return False


def get_validator_schema(schema):
    """Get schema for given data that will work with validator.

    - if field is required without minlength set make sure it's not empty
    - if there are keys with None value - remove them

    :param schema
    """
    validator_schema = {key: val for key, val in schema.items() if val is not None}

    if validator_schema.get('required') and not validator_schema.get('minlength'):
        validator_schema.setdefault('empty', False)

    return validator_schema


class SchemaValidator(Validator):
    def _validate_type_media(self, field, value):
        """Allow type media in schema."""
        pass

    def _validate_type_embed(self, field, value):
        """Allow type media in schema."""
        pass

    def _validate_type_date(self, field, value):
        try:
            datetime.strptime(value or '', '%Y-%m-%dT%H:%M:%S+%f')
        except ValueError:
            self._error(field, DATE_FIELD)

    def _validate_type_picture(self, field, value):
        """Allow type picture in schema."""
        pass

    def _validate_type_text(self, field, value):
        """Validate type text in schema."""
        if value and not isinstance(value, str):
            self._error(field, STRING_FIELD)

    def _validate_mandatory_in_list(self, mandatory, field, value):
        """Validates if all elements from mandatory are presented in the list"""
        for key in mandatory:
            for key_field in mandatory[key]:
                if not check_json(value, key, mandatory[key][key_field]):
                    self._error(key_field, REQUIRED_FIELD)

    def _validate_mandatory_in_dictionary(self, mandatory, field, value):
        """Validates if all elements from mandatory are presented in the dictionary"""
        for key in mandatory:
            if not value.get(key):
                self._error(key, REQUIRED_FIELD)

    def _validate_empty(self, empty, field, value):
        """Original validates only strings, adding a list check."""
        super()._validate_empty(empty, field, value)
        if isinstance(value, list) and not value:
            self._error(field, REQUIRED_FIELD)
        if isinstance(value, str) and value == '<p></p>':  # default value for editor3
            self._error(field, REQUIRED_FIELD)

    def _validate_enabled(self, *args):
        """Ignore ``enabled`` in the schema."""
        pass

    def _validate_place(self, *args):
        """Ignore place."""
        pass

    def _validate_genre(self, *args):
        """Ignore genre."""
        pass

    def _validate_anpa_category(self, *args):
        """Ignore anpa category."""
        pass

    def _validate_subject(self, *args):
        """Ignore subject."""
        pass

    def _validate_company_codes(self, *args):
        """Ignore company codes."""
        pass


class ValidateResource(superdesk.Resource):
    schema = {
        'act': {'type': 'string', 'required': True},
        'type': {'type': 'string', 'required': True},
        'embedded': {'type': 'boolean', 'required': False},
        'validate': {
            'type': 'dict',
            'required': True
        }
    }

    resource_methods = ['POST']
    item_methods = []


class ValidateService(superdesk.Service):

    def create(self, docs, **kwargs):
        for doc in docs:
            test_doc = deepcopy(doc)
            doc['errors'] = self._validate(test_doc, **kwargs)

        return [doc['errors'] for doc in docs]

    def _get_validators(self, doc):
        """Get validators.

        In case there is profile defined for item with respective content type it will
        use its schema for validations, otherwise it will fall back to action/item_type filter.
        """
        extra_field_types = {'text': 'string', 'embed': 'dict', 'date': 'date'}
        profile = doc['validate'].get('profile')
        if profile and (app.config['AUTO_PUBLISH_CONTENT_PROFILE'] or doc['act'] != 'auto_publish'):
            content_type = superdesk.get_resource_service('content_types').find_one(req=None, _id=profile)
            if content_type:
                extra_fields = superdesk.get_resource_service('vocabularies').get_extra_fields()
                schema = content_type.get('schema', {})
                schema['extra'] = {'type': 'dict', 'schema': {}}
                doc['validate'].setdefault('extra', {})  # make sure extra is there so it will validate its fields
                for extra_field in extra_fields:
                    if schema.get(extra_field['_id']) and \
                            extra_field.get('field_type', None) in extra_field_types:
                        rules = schema.pop(extra_field['_id'])
                        rules['type'] = extra_field_types.get(extra_field['field_type'], 'string')
                        schema['extra']['schema'].update({extra_field['_id']: get_validator_schema(rules)})
                        self._populate_extra(doc['validate'], extra_field['_id'])
                content_type['schema'] = schema
                return [content_type]
        lookup = {'act': doc['act'], 'type': doc[ITEM_TYPE]}
        if doc.get('embedded'):
            lookup['embedded'] = doc['embedded']
        else:
            lookup['$or'] = [{'embedded': {'$exists': False}}, {'embedded': False}]
        custom_schema = app.config.get('SCHEMA', {}).get(doc[ITEM_TYPE])
        if custom_schema:
            return [{'schema': custom_schema}]
        return superdesk.get_resource_service('validators').get(req=None, lookup=lookup)

    def _populate_extra(self, doc, schema):
        """Populates the extra field in the document with fields stored in subject. Used
        for user defined vocabularies.

        :param doc: Article to be validated
        :param schema: Vocabulary identifier
        """
        for subject in doc.get('subject', []):
            if subject.get('scheme', '') == schema:
                doc['extra'][schema] = subject.get('qcode', '')

    def _sanitize_fields(self, doc, validator):
        """If maxlength or minlength is specified in the validator then remove any markups from that field

        :param doc: Article to be validated
        :param validator: Validation rule
        :return: updated article
        """
        fields_to_check = ['minlength', 'maxlength']
        item_schema = validator.get('schema', {})
        extra_schema = item_schema.get('extra', {}).get('schema', {})
        schemes_docs = [(item_schema, doc), (extra_schema, doc.get('extra', {}))]
        for schema, content in schemes_docs:
            for field in schema:
                if content.get(field) and schema.get(field) and type(content[field]) is str and \
                        any(k in schema[field] for k in fields_to_check):
                    try:
                        content[field] = get_text(content[field])
                    except (ValueError, TypeError):
                        # fails for json fields like subject, genre
                        pass

    def _get_media_field(self, field_schema, field_associations, doc,):
        """Returns the field name in associations. For multiple valued media fields
        the field asssociations name is different from the field schema name.
        :param field_schema: Field schema name
        :param field_associations: Field associations name
        :param doc: Article to be validated
        """
        if field_associations in doc.get('associations', {}):
            return field_associations
        media_multivalue_field = field_schema + '--'
        for media_field in doc.get('associations', {}):
            if media_field.startswith(media_multivalue_field) and \
                    doc.get('associations', {}).get(media_field):
                return media_field
        return None

    def _process_media(self, doc, validation_schema):
        """If media field(feature media or custom media field) is required it should be present for
        validation on doc not only on associations
        If media_description is required it should be present for validation on doc not only on
        associations->featuremedia (or custom media field)
        :param doc: Article to be validated
        :param schema: Schema to validate the article
        """
        for field_schema in validation_schema:
            field_associations = field_schema if field_schema != 'feature_media' else 'featuremedia'
            media_field = self._get_media_field(field_schema, field_associations, doc)
            if media_field and isinstance(doc['associations'][media_field], MutableMapping):
                doc[field_schema] = doc['associations'][media_field]
                if not doc.get('feature_media', None) is None and 'description_text' in doc['feature_media']:
                    doc['media_description'] = doc['associations']['featuremedia']['description_text']

    def _process_sms(self, doc, schema):
        """Apply the SMS validation to the sms_message value if the document is flagged for SMS
        :param doc:
        :param schema:
        :return:
        """
        if doc.get('flags', {}).get('marked_for_sms', False):
            doc['sms'] = doc.get('sms_message', '')
        else:
            # remove it from the valiadation it is not required
            schema.pop('sms', None)

    def _get_validator_schema(self, validator):
        """Get schema for given validator.

        And make sure there is no `None` value which would raise an exception.
        """
        return {field: get_validator_schema(schema) for field, schema in validator['schema'].items() if schema}

    def _get_vocabulary_display_name(self, vocabulary_id):
        vocabulary = get_resource_service('vocabularies').find_one(req=None, _id=vocabulary_id)
        if vocabulary and 'display_name' in vocabulary:
            return vocabulary['display_name']
        return vocabulary_id

    def _set_default_subject_scheme(self, item):
        if item.get('subject'):
            for subject in item['subject']:
                subject.setdefault('scheme', None)

    def _validate(self, doc, **kwargs):
        use_headline = kwargs and 'headline' in kwargs
        validators = self._get_validators(doc)
        for validator in validators:
            validation_schema = self._get_validator_schema(validator)
            self._sanitize_fields(doc['validate'], validator)
            self._set_default_subject_scheme(doc['validate'])
            self._process_media(doc['validate'], validation_schema)
            self._process_sms(doc['validate'], validation_schema)
            v = SchemaValidator()
            v.allow_unknown = True
            try:
                v.validate(doc['validate'], validation_schema)
            except TypeError as e:
                logger.exception('Invalid validator schema value "%s" for ' % str(e))
            error_list = v.errors
            response = []
            for e in error_list:
                messages = []
                # Ignore dateline if item is corrected because it can't be changed after the item is published
                if doc.get('act', None) == 'correct' and e == 'dateline':
                    continue
                elif doc.get('act', None) == 'kill' and doc['validate'].get('profile', None) and \
                        e in ('headline', 'abstract', 'body_html'):
                    continue
                elif e == 'extra':
                    for field in error_list[e]:
                        display_name = self._get_vocabulary_display_name(field)
                        if 'required' in error_list[e][field]:
                            messages.append(REQUIRED_ERROR.format(display_name))
                        else:
                            messages.append('{} {}'.format(display_name, error_list[e][field]))
                elif error_list[e] == 'required field' or type(error_list[e]) is dict or \
                        type(error_list[e]) is list:
                    messages.append(REQUIRED_ERROR.format(e.upper()))
                elif 'min length is 1' == error_list[e] or 'null value not allowed' in error_list[e]:
                    messages.append(REQUIRED_ERROR.format(e.upper()))
                elif 'min length is' in error_list[e]:
                    messages.append('{} is too short'.format(e.upper()))
                elif 'max length is' in error_list[e]:
                    messages.append('{} is too long'.format(e.upper()))
                else:
                    messages.append('{} {}'.format(e.upper(), error_list[e]))

                for message in messages:
                    if use_headline:
                        headline = '{}: {}'.format(doc['validate'].get('headline',
                                                                       doc['validate'].get('_id')), message)
                        response.append(headline)
                    else:
                        response.append(message)
            return response
        else:
            logger.warn('validator was not found for {}'.format(doc['act']))
            return []



"""Geonames utils."""


def get_geonames_state_qcode(place):
    return 'iso3166-2:%s-%s' % (place.get('country_code'), place.get('state_code'))


def get_geonames_country_qcode(place):
    return 'iso3166-1a2:%s' % place.get('country_code')


def get_geonames_qcode(place):
    if place.get('feature_class', '').upper() == 'A':
        if place.get('state_code'):  # state
            return get_geonames_state_qcode(place)
        else:  # country
            return get_geonames_country_qcode(place)
    return 'geonames:%s' % place.get('code', '0')




import re
import bson

from flask import current_app as app

from .user_mentions import notify_mentioned_users


# client is using @[display name](type:id)
USER_MENTIONS_REGEX = re.compile(r'@\[([^]]+)\]\(user:([a-f0-9]{24})\)')


def handle_inline_mentions(sender, updates, original):
    """Listen to item_update signal and send notifications to new inline user mentions."""
    updated = original.copy()
    updated.update(updates)
    comments = _get_inline_comments(updates)
    for comment in comments:
        if not comment.get('notified'):
            users = _get_mentioned_users(comment)
            if users:
                notify_mentioned_users([{
                    '_id': '',
                    'item': original.get('_id'),
                    'text': _format_comment_text(comment),
                    'mentioned_users': {user: bson.ObjectId(user) for user in users},
                }], app.config.get('CLIENT_URL', ''), item=updated)
            comment['notified'] = True


def _get_mentioned_users(comment):
    return [group[1] for group in USER_MENTIONS_REGEX.findall(comment.get('msg', ''))]


def _format_comment_text(comment):
    def repl(match):
        return match.group(1)
    return USER_MENTIONS_REGEX.sub(repl, comment.get('msg', ''))


def _get_inline_comments(updates):
    try:
        comments = []
        data = (updates['fields_meta']
                       ['body_html']
                       ['draftjsState'][0]
                       ['blocks'][0]
                       ['data'].get('__PUBLIC_API__comments', []))
        for val in data:
            comments.append(val)
            for reply in val.get('replies', []):
                comments.append(reply)
        return comments
    except (KeyError, IndexError):
        return []



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
import json
import itertools

import superdesk
from flask import current_app as app

from superdesk import get_resource_service
from superdesk.errors import SuperdeskApiError
from superdesk.resource import Resource
from superdesk import config
from superdesk.utils import SuperdeskBaseEnum
from bson.objectid import ObjectId
from superdesk.services import BaseService
from superdesk.notification import push_notification
from superdesk.activity import add_activity, ACTIVITY_UPDATE
from superdesk.metadata.item import FAMILY_ID
from eve.utils import ParsedRequest


class DeskTypes(SuperdeskBaseEnum):
    authoring = 'authoring'
    production = 'production'


desks_schema = {
    'name': {
        'type': 'string',
        'required': True,
        'nullable': False,
        'empty': False,
        'iunique': True
    },
    'description': {
        'type': 'string'
    },
    'members': {
        'type': 'list',
        'schema': {
            'type': 'dict',
            'schema': {
                'user': Resource.rel('users', True)
            }
        }
    },
    'incoming_stage': Resource.rel('stages', True),
    'working_stage': Resource.rel('stages', True),
    'content_expiry': {
        'type': 'integer'
    },
    'source': {
        'type': 'string'
    },
    'monitoring_settings': {
        'type': 'list',
        'schema': {
            'type': 'dict',
            'schema': {
                '_id': {
                    'type': 'string',
                    'required': True
                },
                'type': {
                    'type': 'string',
                    'allowed': ['search', 'stage', 'scheduledDeskOutput', 'deskOutput', 'personal'],
                    'required': True
                },
                'max_items': {
                    'type': 'integer',
                    'required': True
                }
            }
        }
    },
    'desk_type': {
        'type': 'string',
        'default': DeskTypes.authoring.value,
        'allowed': DeskTypes.values()
    },
    'desk_metadata': {
        'type': 'dict',
    },

    'content_profiles': {
        'type': 'dict',
    },

    'desk_language': {
        'type': 'string'
    },

    'monitoring_default_view': {
        'type': 'string',
        'allowed': ['', 'list', 'swimlane', 'photogrid'],
        'required': False
    },

    'default_content_profile': Resource.rel('content_types', type='string', nullable=True),
    'default_content_template': Resource.rel('content_templates', nullable=True),
    # name of a Slack channel that may be associated with the desk
    'slack_channel_name': {
        'type': 'string'
    },

    # desk prefered vocabulary items
    'preferred_cv_items': {
        'type': 'dict',
    },
}


def init_app(app):
    endpoint_name = 'desks'
    service = DesksService(endpoint_name, backend=superdesk.get_backend())
    DesksResource(endpoint_name, app=app, service=service)
    endpoint_name = 'user_desks'
    service = UserDesksService(endpoint_name, backend=superdesk.get_backend())
    UserDesksResource(endpoint_name, app=app, service=service)
    endpoint_name = 'sluglines'
    service = SluglineDeskService(endpoint_name, backend=superdesk.get_backend())
    SluglineDesksResource(endpoint_name, app=app, service=service)


superdesk.privilege(name='desks', label='Desk Management', description='User can manage desks.')


class DesksResource(Resource):
    schema = desks_schema
    privileges = {'POST': 'desks', 'PATCH': 'desks', 'DELETE': 'desks'}


class DesksService(BaseService):
    notification_key = 'desk'

    def create(self, docs, **kwargs):
        """Creates new desk.

        Overriding to check if the desk being created has Working and Incoming Stages. If not then Working and Incoming
        Stages would be created and associates them with the desk and desk with the Working and Incoming Stages.
        Also sets desk_type.

        :return: list of desk id's
        """

        for desk in docs:
            stages_to_be_linked_with_desk = []
            stage_service = superdesk.get_resource_service('stages')

            if desk.get('content_expiry') == 0:
                desk['content_expiry'] = app.settings['CONTENT_EXPIRY_MINUTES']

            if 'working_stage' not in desk:
                stages_to_be_linked_with_desk.append('working_stage')
                stage_id = stage_service.create_working_stage()
                desk['working_stage'] = stage_id[0]

            if 'incoming_stage' not in desk:
                stages_to_be_linked_with_desk.append('incoming_stage')
                stage_id = stage_service.create_incoming_stage()
                desk['incoming_stage'] = stage_id[0]

            desk.setdefault('desk_type', DeskTypes.authoring.value)
            super().create([desk], **kwargs)
            for stage_type in stages_to_be_linked_with_desk:
                stage_service.patch(desk[stage_type], {'desk': desk[config.ID_FIELD]})

        return [doc[config.ID_FIELD] for doc in docs]

    def on_created(self, docs):
        for doc in docs:
            push_notification(self.notification_key, created=1, desk_id=str(doc.get(config.ID_FIELD)))
            get_resource_service('users').update_stage_visibility_for_users()

    def on_update(self, updates, original):
        if updates.get('content_expiry') == 0:
            updates['content_expiry'] = None

        if updates.get('desk_type') and updates.get('desk_type') != original.get('desk_type', ''):
            archive_versions_query = {
                '$or': [
                    {'task.last_authoring_desk': str(original[config.ID_FIELD])},
                    {'task.last_production_desk': str(original[config.ID_FIELD])}
                ]
            }

            items = superdesk.get_resource_service('archive_versions').get(req=None, lookup=archive_versions_query)
            if items and items.count():
                raise SuperdeskApiError.badRequestError(
                    message='Cannot update Desk Type as there are article(s) referenced by the Desk.')

    def on_updated(self, updates, original):
        self.__send_notification(updates, original)

    def on_delete(self, desk):
        """Runs on desk delete.

        Overriding to prevent deletion of a desk if the desk meets one of the below conditions:
            1. The desk isn't assigned as a default desk to user(s)
            2. The desk has no content
            3. The desk is associated with routing rule(s)
        """

        as_default_desk = superdesk.get_resource_service('users').get(req=None, lookup={'desk': desk[config.ID_FIELD]})
        if as_default_desk and as_default_desk.count():
            raise SuperdeskApiError.preconditionFailedError(
                message='Cannot delete desk as it is assigned as default desk to user(s).')

        routing_rules_query = {
            '$or': [
                {'rules.actions.fetch.desk': desk[config.ID_FIELD]},
                {'rules.actions.publish.desk': desk[config.ID_FIELD]}
            ]
        }
        routing_rules = superdesk.get_resource_service('routing_schemes').get(req=None, lookup=routing_rules_query)
        if routing_rules and routing_rules.count():
            raise SuperdeskApiError.preconditionFailedError(
                message='Cannot delete desk as routing scheme(s) are associated with the desk')

        archive_versions_query = {
            '$or': [
                {'task.desk': str(desk[config.ID_FIELD])},
                {'task.last_authoring_desk': str(desk[config.ID_FIELD])},
                {'task.last_production_desk': str(desk[config.ID_FIELD])}
            ]
        }

        items = superdesk.get_resource_service('archive_versions').get(req=None, lookup=archive_versions_query)
        if items and items.count():
            raise SuperdeskApiError.preconditionFailedError(
                message='Cannot delete desk as it has article(s) or referenced by versions of the article(s).')

    def delete(self, lookup):
        """
        Overriding to delete stages before deleting a desk
        """

        superdesk.get_resource_service('stages').delete(lookup={'desk': lookup.get(config.ID_FIELD)})
        super().delete(lookup)

    def on_deleted(self, doc):
        desk_user_ids = [str(member['user']) for member in doc.get('members', [])]
        push_notification(self.notification_key,
                          deleted=1,
                          user_ids=desk_user_ids,
                          desk_id=str(doc.get(config.ID_FIELD)))

    def __compare_members(self, original, updates):
        original_members = set([member['user'] for member in original])
        updates_members = set([member['user'] for member in updates])
        added = updates_members - original_members
        removed = original_members - updates_members
        return added, removed

    def __send_notification(self, updates, desk):
        desk_id = desk[config.ID_FIELD]

        if 'members' in updates:
            added, removed = self.__compare_members(desk.get('members', {}), updates['members'])
            if len(removed) > 0:
                push_notification('desk_membership_revoked',
                                  updated=1,
                                  user_ids=[str(item) for item in removed],
                                  desk_id=str(desk_id))

            for added_user in added:
                user = superdesk.get_resource_service('users').find_one(req=None, _id=added_user)
                activity = add_activity(ACTIVITY_UPDATE,
                                        'user {{user}} has been added to desk {{desk}}: Please re-login.',
                                        self.datasource, notify=added, can_push_notification=False,
                                        user=user.get('username'), desk=desk.get('name'))
                push_notification('activity', _dest=activity['recipients'])

            get_resource_service('users').update_stage_visibility_for_users()
        else:
            push_notification(self.notification_key, updated=1, desk_id=str(desk.get(config.ID_FIELD)))

    def apply_desk_metadata(self, updates, original):
        """Apply desk metadata in case it was set in updates and metadata is not set yet on item.

        :param updates: updates to item that should be saved
        :param original: original item version before update
        """
        desk_id = updates.get('task', {}).get('desk', {})
        if not desk_id:
            return
        desk = self.find_one(req=None, _id=desk_id)
        if not desk:
            return
        for key, val in desk.get('desk_metadata', {}).items():
            if key not in updates and key not in original:
                updates[key] = val

    def get_desk_name(self, desk_id):
        """Return the item desk.

        :param desk_id:
        :return dict: desk document
        """
        desk_name = ''
        desk = get_resource_service('desks').find_one(req=None, _id=desk_id)
        if desk:
            desk_name = desk.get('name') or ''

        return desk_name


class UserDesksResource(Resource):
    url = 'users/<regex("[a-f0-9]{24}"):user_id>/desks'
    resource_title = 'user_desks'
    schema = desks_schema
    datasource = {'source': 'desks', 'default_sort': [('name', 1)]}
    resource_methods = ['GET']


class UserDesksService(BaseService):

    def get(self, req, lookup):
        if lookup.get('user_id'):
            lookup['members.user'] = ObjectId(lookup['user_id'])
            del lookup['user_id']
        return super().get(req, lookup)

    def is_member(self, user_id, desk_id):
        # desk = list(self.get(req=None, lookup={'members.user':ObjectId(user_id), '_id': ObjectId(desk_id)}))
        return len(list(self.get(req=None, lookup={'members.user': ObjectId(user_id), '_id': ObjectId(desk_id)}))) > 0


class SluglineDesksResource(Resource):

    url = 'desks/<regex("[a-f0-9]{24}"):desk_id>/sluglines'
    datasource = {'source': 'published',
                  'search_backend': 'elastic',
                  'default_sort': [('slugline.phrase', 1), ("versioncreated", 0)],
                  'elastic_filter': {"and": [{"range": {"versioncreated": {"gte": "now-24H"}}},
                                             {"term": {"last_published_version": True}},
                                             {"term": {"type": "text"}}
                                             ]}}
    resource_methods = ['GET']
    item_methods = []


class SluglineDeskService(BaseService):
    SLUGLINE = 'slugline'
    OLD_SLUGLINES = 'old_sluglines'
    VERSION_CREATED = 'versioncreated'
    HEADLINE = 'headline'
    NAME = 'name'
    PLACE = 'place'
    GROUP = 'group'

    def _get_slugline_with_legal(self, article):
        """If the article is set to be legal adds 'Legal:' prefix for slugline.

        :param article:
        :return:
        """
        is_legal = article.get('flags', {}).get('marked_for_legal', False)
        if is_legal:
            return '{}: {}'.format('Legal', article.get(self.SLUGLINE, ''))
        else:
            return article.get(self.SLUGLINE, '')

    def get(self, req, lookup):
        """Return desk item summary.

        Given the desk the function will return a summary of the sluglines and headlines published from that
        desk in the last 24 hours. Domestic items are grouped together, rest of the world items are group
        by their place names.

        :param req:
        :param lookup:
        :return:
        """
        lookup['task.desk'] = lookup['desk_id']
        lookup.pop('desk_id')
        req.max_results = 1000
        desk_items = super().get(req, lookup)

        # domestic docs
        docs = []
        # rest of the world docs
        row_docs = []
        for item in desk_items:
            slugline = self._get_slugline_with_legal(item)
            headline = item.get(self.HEADLINE)
            versioncreated = item.get(self.VERSION_CREATED)
            placename = 'Domestic'
            # Determine if the item is either domestic or rest of the world
            place = next((place for place in (item.get('place') or [])), None)

            if place and place.get(self.GROUP) == 'Rest Of World':
                row = True
                placename = place.get(self.NAME, 'Domestic')
            else:
                row = False
            # Find if there are other sluglines in this items family
            newer, older_slugline = self._find_other_sluglines(item.get(FAMILY_ID), slugline,
                                                               item.get(self.VERSION_CREATED),
                                                               lookup['task.desk'])
            # there are no newer sluglines than the current one
            if not newer:
                if row:
                    self._add_slugline_to_places(row_docs, placename, slugline, headline, older_slugline,
                                                 versioncreated)
                else:
                    self._add_slugline_to_places(docs, placename, slugline, headline, older_slugline, versioncreated)

        docs.extend(row_docs)
        items = []

        # group by place and sort all items of a place by versioncreated.
        for key, group in itertools.groupby(docs, lambda x: x['name']):
            items.append({'place': key, 'items': sorted(group, key=lambda k: k['versioncreated'], reverse=True)})

        desk_items.docs = items
        return desk_items

    def _add_slugline_to_places(self, places, placename, slugline, headline, old_sluglines, versioncreated):
        """Add slugline to places.

        Append a dictionary to the list, with place holders for the place name and slugline if they are already
        present.

        :param places:
        :param placename:
        :param slugline:
        :param headline:
        :param is_legal:
        :param old_sluglines:
        :param versioncreated:
        :return:
        """
        places.append({self.NAME: placename,
                       self.SLUGLINE: slugline if not any(self._get_slugline_with_legal(p).lower() == slugline.lower()
                                                          for p in places) else '-',
                       self.HEADLINE: headline,
                       self.OLD_SLUGLINES: old_sluglines,
                       self.VERSION_CREATED: versioncreated})

    def _find_other_sluglines(self, family_id, slugline, versioncreated, desk_id):
        """Find other sluglines.

        This function given a family_id will return a tuple with the first value true if there is
         a more recent story in the family, the second value in the tuple will be a list of any sluglines
         that might exist for the family that are different to the one passed.

        :param family_id:
        :param slugline:
        :param versioncreated:
        :param desk_id:
        :return: A tuple as described above
        """
        older_sluglines = []
        req = ParsedRequest()
        query = {
            'query': {
                'filtered': {
                    'filter': {
                        'and': [
                            {'term': {'family_id': family_id}},
                            {'term': {'task.desk': desk_id}},
                        ]
                    }
                }
            }
        }
        req.args = {'source': json.dumps(query), 'aggregations': 0}
        family = superdesk.get_resource_service('published').get(req=req, lookup=None)
        for member in family:
            member_slugline = self._get_slugline_with_legal(member)
            if member_slugline.lower() != slugline.lower():
                if member.get('versioncreated') < versioncreated:
                    if member_slugline not in older_sluglines:
                        older_sluglines.append(member_slugline)
                else:
                    return (True, [])
        return (False, older_sluglines)


def remove_profile_from_desks(item):
    """Removes the profile data from desks that are using the profile

    :param item: deleted content profile
    """
    req = ParsedRequest()
    desks = list(superdesk.get_resource_service('desks').get(req=req, lookup={}))
    for desk in desks:
        if desk.get('default_content_profile') == str(item.get(config.ID_FIELD)):
            desk['default_content_profile'] = None
            superdesk.get_resource_service('desks').patch(desk[config.ID_FIELD], desk)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import os
import shutil
from datetime import timedelta, datetime

from lxml import etree
from superdesk.errors import IngestFileError, ParserError, ProviderError
from superdesk.io.registry import register_feeding_service
from superdesk.io.feed_parsers import XMLFeedParser
from superdesk.io.feeding_services import FeedingService
from superdesk.notification import push_notification
from superdesk.utc import utc, utcnow
from superdesk.utils import get_sorted_files, FileSortAttributes

logger = logging.getLogger(__name__)


class FileFeedingService(FeedingService):
    """
    Feeding Service class which can read the configured local file system for article(s).
    """

    NAME = 'file'

    ERRORS = [
        ParserError.IPTC7901ParserError().get_error_description(),
        ParserError.nitfParserError().get_error_description(),
        ParserError.newsmlOneParserError().get_error_description(),
        ProviderError.ingestError().get_error_description(),
        ParserError.parseFileError().get_error_description()
    ]

    label = 'File feed'

    fields = [
        {
            'id': 'path', 'type': 'text', 'label': 'Server Folder',
            'placeholder': 'path to folder', 'required': True,
            'errors': {3003: 'Path not found on server.', 3004: 'Path should be directory.'}
        }
    ]

    def _test(self, provider):
        path = provider.get('config', {}).get('path', None)
        if not os.path.exists(path):
            raise IngestFileError.notExistsError()
        if not os.path.isdir(path):
            raise IngestFileError.isNotDirError()

    def _update(self, provider, update):
        self.provider = provider
        self.path = provider.get('config', {}).get('path', None)

        if not self.path:
            logger.warn('File Feeding Service {} is configured without path. Please check the configuration'
                        .format(provider['name']))
            return []

        registered_parser = self.get_feed_parser(provider)
        for filename in get_sorted_files(self.path, sort_by=FileSortAttributes.created):
            try:
                last_updated = None
                file_path = os.path.join(self.path, filename)
                if os.path.isfile(file_path):
                    stat = os.lstat(file_path)
                    last_updated = datetime.fromtimestamp(stat.st_mtime, tz=utc)

                    if self.is_latest_content(last_updated, provider.get('last_updated')):
                        if isinstance(registered_parser, XMLFeedParser):
                            with open(file_path, 'rb') as f:
                                xml = etree.parse(f)
                                parser = self.get_feed_parser(provider, xml.getroot())
                                item = parser.parse(xml.getroot(), provider)
                        else:
                            parser = self.get_feed_parser(provider, file_path)
                            item = parser.parse(file_path, provider)

                        self.after_extracting(item, provider)
                        self.move_file(self.path, filename, provider=provider, success=True)

                        if isinstance(item, list):
                            yield item
                        else:
                            yield [item]
                    else:
                        self.move_file(self.path, filename, provider=provider, success=True)
            except Exception as ex:
                if last_updated and self.is_old_content(last_updated):
                    self.move_file(self.path, filename, provider=provider, success=False)
                raise ParserError.parseFileError('{}-{}'.format(provider['name'], self.NAME), filename, ex, provider)

        push_notification('ingest:update')

    def after_extracting(self, article, provider):
        """Sub-classes should override this method if something needs to be done to the given article.

        For example, if the article comes from DPA provider the system needs to derive dateline
        from the properties in the article.

        Invoked after parser parses the article received from the provider.

        :param article: dict having properties that can be saved into ingest collection
        :type article: dict
        :param provider: dict - Ingest provider details to which the current directory has been configured
        :type provider: dict :py:class: `superdesk.io.ingest_provider_model.IngestProviderResource`
        """
        pass

    def move_file(self, file_path, filename, provider, success=True):
        """Move the files from the current directory to the _Processed if successful, else _Error if unsuccessful.

        Creates _Processed and _Error directories within current directory if they don't exist.

        :param file_path: str - current directory location
        :param filename: str - file name in the current directory to move
        :param provider: dict - Ingest provider details to which the current directory has been configured
        :param success: bool - default value is True. When True moves to _Processed directory else _Error directory.
        :raises IngestFileError.folderCreateError() if creation of _Processed or _Error directories fails
        :raises IngestFileError.fileMoveError() if failed to move the file pointed by filename
        """

        try:
            if not os.path.exists(os.path.join(file_path, "_PROCESSED/")):
                os.makedirs(os.path.join(file_path, "_PROCESSED/"))
            if not os.path.exists(os.path.join(file_path, "_ERROR/")):
                os.makedirs(os.path.join(file_path, "_ERROR/"))
        except Exception as ex:
            raise IngestFileError.folderCreateError(ex, provider)

        try:
            if success:
                shutil.copy2(os.path.join(file_path, filename), os.path.join(file_path, "_PROCESSED/"))
            else:
                shutil.copy2(os.path.join(file_path, filename), os.path.join(file_path, "_ERROR/"))
        except Exception as ex:
            raise IngestFileError.fileMoveError(ex, provider)
        finally:
            os.remove(os.path.join(file_path, filename))

    def is_latest_content(self, last_updated, provider_last_updated=None):
        """
        Parse file only if it's not older than provider last update -10m
        """

        if not provider_last_updated:
            provider_last_updated = utcnow() - timedelta(days=7)

        return provider_last_updated - timedelta(minutes=10) < last_updated

    def is_old_content(self, last_updated):
        """Test if file is old so it wouldn't probably work in is_latest_content next time.

        Such files can be moved to `_ERROR` folder, it wouldn't be ingested anymore.

        :param last_updated: file last updated datetime
        """
        return last_updated < utcnow() - timedelta(minutes=10)


register_feeding_service(FileFeedingService)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from eve.utils import config
from copy import deepcopy

from superdesk import get_resource_service
from superdesk.resource import Resource
from superdesk.services import BaseService
from apps.archive.common import ITEM_UPDATE, get_user, ITEM_CREATE
from superdesk.metadata.item import CONTENT_STATE, ITEM_STATE
from superdesk.utc import utcnow

log = logging.getLogger(__name__)

fields_to_remove = ['_id', '_etag', 'versioncreator', 'originalcreator', 'versioncreated',
                    '_current_version', 'version', '_updated', 'lock_session', 'lock_user', 'lock_time', 'lock_action',
                    'force_unlock', '_created', 'guid', 'family_id', 'firstcreated', 'original_creator']


class ArchiveHistoryResource(Resource):
    endpoint_name = 'archive_history'
    resource_methods = ['GET']
    item_methods = ['GET']
    schema = {
        'item_id': {'type': 'string'},
        'user_id': Resource.rel('users', True),
        'operation': {'type': 'string'},
        'update': {'type': 'dict', 'nullable': True},
        'version': {'type': 'integer'},
        'original_item_id': {'type': 'string'}
    }

    mongo_indexes = {'item_id': ([('item_id', 1)], {'background': True})}


class ArchiveHistoryService(BaseService):

    def on_item_updated(self, updates, original, operation=None):
        item = deepcopy(original)
        if updates:
            item.update(updates)

        self._save_history(item, updates, operation or ITEM_UPDATE)

    def on_item_deleted(self, doc):
        lookup = {'item_id': doc[config.ID_FIELD]}
        self.delete(lookup=lookup)

    def get_user_id(self, item):
        user = get_user()
        if user:
            return user.get(config.ID_FIELD)

    def _save_history(self, item, update, operation):
        # in case of auto-routing, if the original_creator exists in our database
        # then create item create record in the archive history.
        if item.get(ITEM_STATE) == CONTENT_STATE.ROUTED and item.get('original_creator') \
                and not item.get('original_id'):
            user = get_resource_service('users').find_one(req=None, _id=item.get('original_creator'))
            firstcreated = item.get('firstcreated', utcnow())

            if user:
                history = {
                    'item_id': item[config.ID_FIELD],
                    'user_id': user.get(config.ID_FIELD),
                    'operation': ITEM_CREATE,
                    'update': self._remove_unwanted_fields(update, item),
                    'version': item.get(config.VERSION, 1),
                    '_created': firstcreated,
                    '_updated': firstcreated
                }

                self.post([history])

        history = {
            'item_id': item[config.ID_FIELD],
            'user_id': self.get_user_id(item),
            'operation': operation,
            'update': self._remove_unwanted_fields(update, item),
            'version': item.get(config.VERSION, 1)
        }

        self.post([history])

    def _remove_unwanted_fields(self, update, original):
        if update:
            update_copy = deepcopy(update)
            for field in fields_to_remove:
                update_copy.pop(field, None)

            if original.get('sms_message') == update_copy.get('sms_message'):
                update_copy.pop('sms_message', None)

            return update_copy



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : superdesk
# Creation: 2017-08-14 11:47

from superdesk.commands.data_updates import DataUpdate
from superdesk import get_resource_service
from eve.utils import config, app
from eve.io.mongo.mongo import create_index
from superdesk.audit.commands import PurgeAudit


class DataUpdate(DataUpdate):
    resource = 'audit'

    def forwards(self, mongodb_collection, mongodb_database):
        for audit in mongodb_collection.find({'resource': {'$in': PurgeAudit.item_resources}}):
            audit_id = get_resource_service(self.resource)._extract_doc_id(audit.get('extra'))
            print(mongodb_collection.update({'_id': audit.get(config.ID_FIELD)},
                                            {'$set': {
                                                'audit_id': audit_id
                                            }}))
        try:
            create_index(app=app, resource=self.resource, name='audit_id', list_of_keys=[('audit_id', 1)],
                         index_options={'background': True})
        except Exception:
            print('create index failed')

    def backwards(self, mongodb_collection, mongodb_database):
        print(mongodb_collection.update({},
                                        {'$unset': {'audit_id': []}},
                                        upsert=False, multi=True))



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : superdesk
# Creation: 2017-02-14 14:53

from superdesk.commands.data_updates import DataUpdate
from superdesk import get_resource_service


class DataUpdate(DataUpdate):

    resource = 'vocabularies'
    product_types = {
        '_id': 'product_types',
        'display_name': 'Product Types',
        'type': 'unmanageable',
        'items': [
            {'is_active': True, 'name': 'API', 'qcode': 'api'},
            {'is_active': True, 'name': 'Direct', 'qcode': 'direct'},
            {'is_active': True, 'name': 'Both', 'qcode': 'both'}
        ]
    }

    def forwards(self, mongodb_collection, mongodb_database):
        product_types = get_resource_service(self.resource).find_one(req=None, _id='product_types')
        if product_types:
            print('Product Types vocabulary already exists in the system.')
            return
        get_resource_service(self.resource).post([self.product_types])

    def backwards(self, mongodb_collection, mongodb_database):
        get_resource_service(self.resource).delete_action({'_id': 'product_types'})



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : Jérôme
# Creation: 2018-04-04 16:55

from superdesk.commands.data_updates import DataUpdate


class DataUpdate(DataUpdate):

    resource = 'archive'  # will use multiple resources, keeping this here so validation passes

    def forwards(self, mongodb_collection, mongodb_database):
        for resource in ['archive', 'archive_autosave', 'published']:

            collection = mongodb_database[resource]

            for item in collection.find({'editor_state': {'$exists': True}}):
                state = item['editor_state']
                fields_meta = {'body_html': {'draftjsState': state}}
                print(collection.update({'_id': item['_id']}, {
                    '$set': {
                        'fields_meta': fields_meta
                    },
                    '$unset': {
                        'editor_state': ''
                    }
                }))

    def backwards(self, mongodb_collection, mongodb_database):
        for resource in ['archive', 'archive_autosave', 'published']:

            collection = mongodb_database[resource]

            for item in collection.find({'fields_meta': {'$exists': True}}):
                state = item['fields_meta']['body_html']['draftjsState']
                print(collection.update({'_id': item['_id']}, {
                    '$set': {
                        'editor_state': state
                    },
                    '$unset': {
                        'fields_meta': ''
                    }
                }))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import feedparser
import requests

from calendar import timegm
from collections import namedtuple
from datetime import datetime

from superdesk.errors import IngestApiError, ParserError
from superdesk.io.registry import register_feeding_service
from superdesk.io.feeding_services import FeedingService
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, GUID_TAG
from superdesk.utils import merge_dicts
from superdesk.metadata.utils import generate_guid, generate_tag, generate_tag_from_url

from urllib.parse import quote as urlquote, urlsplit, urlunsplit


utcfromtimestamp = datetime.utcfromtimestamp


class RSSFeedingService(FeedingService):
    """
    Feeding service for providing feeds received in RSS 2.0 format.

    (NOTE: it should also work with other syndicated feeds formats, too, since
    the underlying parser supports them, but for our needs RSS 2.0 is assumed)
    """

    NAME = 'rss'

    ERRORS = [IngestApiError.apiAuthError().get_error_description(),
              IngestApiError.apiNotFoundError().get_error_description(),
              IngestApiError.apiGeneralError().get_error_description(),
              ParserError.parseMessageError().get_error_description()]

    label = 'RSS'

    fields = [
        {
            'id': 'url', 'type': 'text', 'label': 'Host',
            'placeholder': 'RSS Feed URL', 'required': True,
            'errors': {4001: 'Connection timed out.', 4006: 'URL not found.',
                       4009: 'Can\'t connect to host.', 1001: 'Can\'t parse the RSS.'}
        },
        {
            'id': 'auth_required', 'type': 'boolean', 'label': 'Requires Authentication',
            'placeholder': 'Requires Authentication', 'required': False
        },
        {
            'id': 'username', 'type': 'text', 'label': 'Username',
            'placeholder': 'Username', 'required_expression': '{auth_required}',
            'show_expression': '{auth_required}'
        },
        {
            'id': 'password', 'type': 'password', 'label': 'Password',
            'placeholder': 'Password', 'required_expression': '{auth_required}',
            'show_expression': '{auth_required}'
        },
        {
            'id': 'field_aliases', 'type': 'mapping', 'label': 'Content Field Aliases',
            'add_mapping_label': 'Add alias', 'remove_mapping_label': 'Remove',
            'empty_label': 'No field aliases defined.',
            'first_field_options': {
                'label': 'Content Field Name',
                'values': ['body_text', 'guid', 'published_parsed', 'summary', 'title', 'updated_parsed']
            },
            'second_field_options': {
                'label': 'Field Alias',
                'placeholder': 'Enter field alias'
            }
        }
    ]

    parser_restricted_values = None

    field_groups = {'auth_data': {'label': 'Authentication Info', 'fields': ['username', 'password']}}

    ItemField = namedtuple('ItemField', ['name', 'name_in_data', 'type'])

    item_fields = [
        ItemField('guid', 'guid', str),
        ItemField('uri', 'guid', str),
        ItemField('firstcreated', 'published_parsed', datetime),
        ItemField('versioncreated', 'updated_parsed', datetime),
        ItemField('headline', 'title', str),
        ItemField('abstract', 'summary', str),
        ItemField('body_html', 'body_text', str),
        ItemField('byline', 'author', str),
    ]
    """A list of fields that items created from the ingest data should contain.

    Each list item is a named tuple with the following three attribues:

    * name - the name of the field (attribute) in the resulting ingest item
    * name_in_data - the expected name of the data field in the retrieved
        ingest data (this can be overriden by providing a field name alias)
    * type - field's data type
    """

    IMG_MIME_TYPES = (
        'image/gif',
        'image/jpeg',
        'image/png',
        'image/tiff',
    )
    """
    Supported MIME types for ingesting external images referenced by the
    RSS entries.
    """

    IMG_FILE_SUFFIXES = ('.gif', '.jpeg', '.jpg', '.png', '.tif', '.tiff')
    """
    Supported image filename extensions for ingesting (used for the
    <media:thumbnail> tags - they lack the "type" attribute).
    """

    def __init__(self):
        super().__init__()
        self.auth_info = None

    def prepare_href(self, url, mimetype=None):
        """Prepare a link to an external resource (e.g. an image file).

        It can be directly used by the ingest machinery for fetching it.

        If provider requires authentication, basic HTTP authentication info is
        added to the given url, otherwise it is returned unmodified.

        :param str url: the original URL as extracted from an RSS entry

        :return: prepared URL
        :rtype: str
        """
        if self.auth_info:
            userinfo_part = '{}:{}@'.format(
                urlquote(self.auth_info['username']),
                urlquote(self.auth_info['password'])
            )
            scheme, netloc, path, query, fragment = urlsplit(url)
            netloc = userinfo_part + netloc
            url = urlunsplit((scheme, netloc, path, query, fragment))

        return url

    def _test(self, provider):
        """Test connection."""
        config = provider.get('config', {})
        xml = self._fetch_data(config, provider)
        data = feedparser.parse(xml)
        if data.bozo:
            raise ParserError.parseMessageError(data.bozo_exception, provider)

    def _update(self, provider, update):
        """
        Check data provider for data updates and returns new items (if any).

        :param provider: data provider instance
        :return: a list containing a list of new content items
        :rtype: list

        :raises IngestApiError: if data retrieval error occurs
        :raises ParserError: if retrieved RSS data cannot be parsed
        """
        config = provider.get('config', {})
        xml_data = self._fetch_data(config, provider)

        try:
            data = feedparser.parse(xml_data)
        except Exception as ex:
            raise ParserError.parseMessageError(ex, provider, data=xml_data)

        # If provider last updated time is not available, set it to 1.1.1970
        # so that it will be recognized as "not up to date".
        # Also convert it to a naive datetime object (removing tzinfo is fine,
        # because it is in UTC anyway)
        t_provider_updated = provider.get('last_updated', utcfromtimestamp(0))
        t_provider_updated = t_provider_updated.replace(tzinfo=None)

        new_items = []
        field_aliases = config.get('field_aliases')

        for entry in data.entries:
            try:
                t_entry_updated = utcfromtimestamp(timegm(entry.updated_parsed))
                if t_entry_updated <= t_provider_updated:
                    continue
            except (AttributeError, TypeError):
                # missing updated info, so better ingest it
                pass

            item = self._create_item(entry, field_aliases, provider.get('source', None))
            self.add_timestamps(item)

            # If the RSS entry references any images, create picture items from
            # them and create a package referencing them and the entry itself.
            # If there are no image references, treat entry as a simple text
            # item, even if it might reference other media types, e.g. videos.
            image_urls = self._extract_image_links(entry)
            if image_urls:
                image_items = self._create_image_items(image_urls, item)
                new_items.extend(image_items)
                new_items.append(item)
                item = self._create_package(item, image_items)

            new_items.append(item)

        return [new_items]

    def _fetch_data(self, config, provider):
        """Fetch the latest feed data.

        :param dict config: RSS resource configuration
        :param provider: data provider instance, needed as an argument when
            raising ingest errors
        :return: fetched RSS data
        :rtype: str

        :raises IngestApiError: if fetching data fails for any reason
            (e.g. authentication error, resource not found, etc.)
        """
        url = config['url']

        if config.get('auth_required', False):
            auth = (config.get('username'), config.get('password'))
            self.auth_info = {
                'username': config.get('username', ''),
                'password': config.get('password', '')
            }
        else:
            auth = None

        try:
            response = requests.get(url, auth=auth, timeout=30)
        except requests.exceptions.ConnectionError as err:
            raise IngestApiError.apiConnectionError(exception=err)
        except requests.exceptions.RequestException as err:
            raise IngestApiError.apiURLError(exception=err)

        if response.ok:
            return response.content
        else:
            if response.status_code in (401, 403):
                raise IngestApiError.apiAuthError(
                    Exception(response.reason), provider)
            elif response.status_code == 404:
                raise IngestApiError.apiNotFoundError(
                    Exception(response.reason), provider)
            else:
                raise IngestApiError.apiGeneralError(
                    Exception(response.reason), provider)

    def _extract_image_links(self, rss_entry):
        """Extract URLs of all images referenced by the given RSS entry.

        Images can be referenced via `<enclosure>`, `<media:thumbnail>` or
        `<media:content>` RSS tag and must be listed among the allowed image
        types. All other links to external media are ignored.

        Duplicate URLs are omitted from the result.

        :param rss_entry: parsed RSS item (entry)
        :type rss_entry: :py:class:`feedparser.FeedParserDict`

        :return: a list of all unique image URLs found (as strings)
        """
        img_links = set()

        for link in getattr(rss_entry, 'links', []):
            if link.get('type') in self.IMG_MIME_TYPES:
                img_links.add(link['href'])

        for item in getattr(rss_entry, 'media_thumbnail', []):
            url = item.get('url', '')
            if url.endswith(self.IMG_FILE_SUFFIXES):
                img_links.add(url)

        for item in getattr(rss_entry, 'media_content', []):
            if item.get('type') in self.IMG_MIME_TYPES:
                img_links.add(item['url'])

        return list(img_links)

    def _create_item(self, data, field_aliases=None, source='source'):
        """Create a new content item from RSS feed data.

        :param dict data: parsed data of a single feed entry
        :param field_aliases: (optional) field name aliases. Used for content
             fields that are named differently in retrieved data.
        :type field_aliases: list of {field_name: alias} dictionaries or None
        :param str source: the source of provider

        :return: created content item
        :rtype: dict
        """
        if field_aliases is None:
            field_aliases = {}
        else:
            field_aliases = merge_dicts(field_aliases)
        aliased_fields = set(field_aliases.values())

        item = dict(type=CONTENT_TYPE.TEXT)

        # Only consider fields that are not used as an alias (i.e. used to
        # populate another field) - unless those fields have their own
        # aliases, too.
        # The idea is that if e.g. the main text field is aliased to use the
        # parsed data's summary field, that summary should not be used to
        # populate the field it was originally meant for.
        fields_to_consider = (
            f for f in self.item_fields
            if (f.name_in_data not in aliased_fields) or
               (f.name_in_data in aliased_fields and
                f.name_in_data in field_aliases)
        )

        utc_now = datetime.utcnow()
        for field in fields_to_consider:
            data_field_name = field_aliases.get(
                field.name_in_data, field.name_in_data
            )
            field_value = data.get(data_field_name)

            if (field.type is datetime) and field_value:
                field_value = utcfromtimestamp(timegm(field_value))
                field_value = utc_now if field_value > utc_now else field_value

            item[field.name] = field_value

            # Some feeds use <content:encoded> tag for storing the main content,
            # and that tag is parsed differently. If the body_html has not been
            # found in its default data field and is not aliased, try to
            # populate it using the aforementioned content field as a fallback.
            if (
                field.name == 'body_html' and
                not field_value and
                field.name_in_data not in field_aliases
            ):
                try:
                    item['body_html'] = data.content[0].value
                except Exception:
                    pass  # content either non-existent or parsed differently

        if not data.get('guidislink') and data.get('link'):
            item['uri'] = data['link']
            scheme, netloc, path, query, fragment = urlsplit(item['uri'])
            if data.get('guid'):
                item['guid'] = generate_tag(domain=netloc, id=data.get('guid'))
            else:
                item['guid'] = generate_tag_from_url(data['link'])

        if item.get('uri', None):
            if not item.get('body_html', None):
                item['body_html'] = ''
            item['body_html'] = '<p><a href="%s" target="_blank">%s</a></p>' % (item['uri'], source) + item['body_html']

        item['dateline'] = {
            'source': source,
            'date': item.get('firstcreated', item.get('versioncreated'))
        }

        return item

    def _create_image_items(self, image_links, text_item):
        """Create a list of picture items that represent the external images located on given URLs.

        Each created item's `firstcreated` and `versioncreated` fields are set
        to the same value as the values of these fields in `text_item`.

        :param iterable image_links: list of image URLs
        :param dict text_item: the "main" text item the images are related to

        :return: list of created image items (as dicts)
        """
        image_items = []

        for image_url in image_links:
            img_item = {
                'guid': generate_guid(type=GUID_TAG),
                ITEM_TYPE: CONTENT_TYPE.PICTURE,
                'firstcreated': text_item.get('firstcreated'),
                'versioncreated': text_item.get('versioncreated'),
                'renditions': {
                    'baseImage': {
                        'href': image_url
                    }
                }
            }
            image_items.append(img_item)

        return image_items

    def _create_package(self, text_item, image_items):
        """Create a new content package from given content items.

        The package's `main` group contains only the references to given items,
        not the items themselves. In the list of references, the reference to
        the text item preceeds the references to image items.

        Package's `firstcreated` and `versioncreated` fields are set to values
        of these fields in `text_item`, and the `headline` is copied as well.

        :param dict text_item: item representing the text content
        :param list image_items: list of items (dicts) representing the images
            related to the text content
        :return: the created content package
        :rtype: dict
        """
        package = {
            ITEM_TYPE: CONTENT_TYPE.COMPOSITE,
            'guid': generate_guid(type=GUID_TAG),
            'firstcreated': text_item['firstcreated'],
            'versioncreated': text_item['versioncreated'],
            'headline': text_item.get('headline', ''),
            'groups': [
                {
                    'id': 'root',
                    'role': 'grpRole:NEP',
                    'refs': [{'idRef': 'main'}],
                }, {
                    'id': 'main',
                    'role': 'main',
                    'refs': [],
                }
            ]
        }

        item_references = package['groups'][1]['refs']
        item_references.append({'residRef': text_item['guid']})

        for image in image_items:
            item_references.append({'residRef': image['guid']})

        return package


register_feeding_service(RSSFeedingService)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2016 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk


class WorkqueueResource(superdesk.Resource):
    endpoint_name = 'workqueue'
    datasource = {'source': 'archive'}
    schema = {
        'lock_user': superdesk.Resource.rel('users')
    }
    item_methods = ['GET']


class WorkqueueService(superdesk.Service):

    pass



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from flask import request
from superdesk.resource import Resource, build_custom_hateoas
from superdesk.metadata.utils import item_url
from .common import get_user, get_auth, CUSTOM_HATEOAS
from superdesk.services import BaseService
from apps.common.components.utils import get_component
from apps.item_lock.components.item_lock import ItemLock


def _update_returned_document(doc, item):
    doc.clear()
    doc.update(item)
    build_custom_hateoas(CUSTOM_HATEOAS, doc)
    return [doc['_id']]


class ArchiveLockResource(Resource):
    endpoint_name = 'archive_lock'
    url = 'archive/<{0}:item_id>/lock'.format(item_url)
    schema = {'lock_action': {'type': 'string'}}
    datasource = {'source': 'archive'}
    resource_methods = ['GET', 'POST']
    resource_title = endpoint_name
    privileges = {'POST': 'archive'}


class ArchiveLockService(BaseService):

    def create(self, docs, **kwargs):
        user = get_user(required=True)
        auth = get_auth()
        item_id = request.view_args['item_id']
        lock_action = docs[0].get('lock_action', 'edit')
        item = get_component(ItemLock).lock({'_id': item_id}, user['_id'], auth['_id'], lock_action)
        return _update_returned_document(docs[0], item)


class ArchiveUnlockResource(Resource):
    endpoint_name = 'archive_unlock'
    url = 'archive/<{0}:item_id>/unlock'.format(item_url)
    schema = {'lock_user': {'type': 'string'}}
    datasource = {'source': 'archive'}
    resource_methods = ['GET', 'POST']
    resource_title = endpoint_name


class ArchiveUnlockService(BaseService):

    def create(self, docs, **kwargs):
        user = get_user(required=True)
        auth = get_auth()
        item_id = request.view_args['item_id']
        item = get_component(ItemLock).unlock({'_id': item_id}, user['_id'], auth['_id'], None)

        if item is None:
            # version 1 item must have been deleted by now
            return [0]

        return _update_returned_document(docs[0], item)



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : Jérôme
# Creation: 2017-10-23 15:24

from superdesk.commands.data_updates import DataUpdate


class DataUpdate(DataUpdate):

    resource = 'vocabularies'

    def forwards(self, mongodb_collection, mongodb_database):
        print(mongodb_collection.update_many({'_id': {'$in': ['author_roles', 'job_titles']}},
                                             {'$set': {
                                                 'unique_field': "qcode"
                                             }}))

    def backwards(self, mongodb_collection, mongodb_database):
        print(mongodb_collection.update_many({'_id': {'$in': ['author_roles', 'job_titles']}},
                                             {'$unset': {
                                                 'unique_field': "qcode"
                                             }}))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import requests
import re

from datetime import datetime

from superdesk.errors import IngestApiError, ParserError
from superdesk.io.registry import register_feeding_service
from superdesk.io.feeding_services import FeedingService

utcfromtimestamp = datetime.utcfromtimestamp


class BBCLDRSFeedingService(FeedingService):
    """
    Feeding Service class for reading BBC's Local Democracy Reporting Service
    """

    # Following the api spec at https://docs.ldrs.org.uk/

    NAME = 'bbc_ldrs'
    ERRORS = [IngestApiError.apiAuthError().get_error_description(),
              IngestApiError.apiNotFoundError().get_error_description(),
              IngestApiError.apiGeneralError().get_error_description(),
              ParserError.parseMessageError().get_error_description()]

    label = 'BBC Local Democracy Reporter Service'

    fields = [
        {
            'id': 'url', 'type': 'text', 'label': 'LDRS URL',
            'placeholder': 'LDRS URL', 'required': True,
            'default': 'https://api.ldrs.org.uk/v1/item'
        },
        {
            'id': 'api_key', 'type': 'text', 'label': 'API Key',
            'placeholder': 'API Key', 'required': True,
            'default': ''
        }
    ]

    def __init__(self):
        super().__init__()

    def _test(self, provider):
        config = provider.get('config', {})
        url = config['url']
        api_key = config['api_key']

        # limit the data to a single article and filter out all article fields
        # to save bandwidth
        params = {'limit': 1, 'fields': 'id'}
        headers = {'apikey': api_key}

        try:
            response = requests.get(url, params=params, headers=headers, timeout=30)
        except requests.exceptions.ConnectionError as err:
            raise IngestApiError.apiConnectionError(exception=err)

        if not response.ok:
            if response.status_code == 404:
                raise IngestApiError.apiNotFoundError(
                    Exception(response.reason), provider)
            else:
                raise IngestApiError.apiGeneralError(
                    Exception(response.reason), provider)

    def _update(self, provider, update):
        config = provider.get('config', {})
        json_items = self._fetch_data(config, provider)
        parsed_items = []

        for item in json_items:
            try:
                parser = self.get_feed_parser(provider, item)
                parsed_items.append(parser.parse(item))
            except Exception as ex:
                raise ParserError.parseMessageError(ex, provider, data=item)

        return parsed_items

    def _fetch_data(self, config, provider):
        url = config['url']
        api_key = config['api_key']

        last_update = provider.get('last_updated', utcfromtimestamp(0)).strftime('%Y-%m-%dT%H:%M:%S')

        # Results are pagified so we'll read this many at a time
        offset_jump = 10

        params = {'start': last_update, 'limit': offset_jump}
        headers = {'apikey': api_key}

        items = []

        offset = 0
        while True:
            params['offset'] = offset

            try:
                response = requests.get(url, params=params, headers=headers, timeout=30)
            except requests.exceptions.ConnectionError as err:
                raise IngestApiError.apiConnectionError(exception=err)

            if response.ok:
                # The total number of results are given to us in json, get them
                # via a regex to read the field so we don't have to convert the
                # whole thing to json pointlessly
                item_ident = re.search('\"total\": *[0-9]*', response.text).group()
                results_str = re.search('[0-9]+', item_ident).group()

                if results_str is None:
                    raise IngestApiError.apiGeneralError(
                        Exception(response.text), provider)

                num_results = int(results_str)

                if num_results > 0:
                    items.append(response.text)

                if offset >= num_results:
                    return items

                offset += offset_jump
            else:
                if re.match('Error: No API Key provided', response.text):
                    raise IngestApiError.apiAuthError(
                        Exception(response.text), provider)
                elif response.status_code == 404:
                    raise IngestApiError.apiNotFoundError(
                        Exception(response.reason), provider)
                else:
                    raise IngestApiError.apiGeneralError(
                        Exception(response.reason), provider)

        return items


register_feeding_service(BBCLDRSFeedingService)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
from lxml import etree
from superdesk.io.feed_parsers.scoop_newsml_2_0 import ScoopNewsMLTwoFeedParser
from superdesk.tests import TestCase


class ScoopTestCase(TestCase):

    vocab = [{'_id': 'locators', 'items': [{
        "is_active": True,
        "name": "NZ",
        "world_region": "Oceania",
        "country": "New Zealand",
        "state": "",
        "qcode": "NZ",
        "group": "Rest Of World"
    }]}]

    def setUp(self):
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, '../fixtures', self.filename))
        provider = {'name': 'Test'}
        with self.app.app_context():
            self.app.data.insert('vocabularies', self.vocab)
        with open(fixture, 'rb') as f:
            parser = ScoopNewsMLTwoFeedParser()
            self.xml = etree.parse(f)
            self.item = parser.parse(self.xml.getroot(), provider)


class ScoopTestCase(ScoopTestCase):
    filename = 'scoop.xml'

    def test_content(self):
        self.assertEqual(self.item[0].get('headline'),
                         "Dick Smith receivers install Don Grover as acting CEO as Abboud " +
                         "quits; retailer put up for sale")
        self.assertEqual(self.item[0].get('guid'), 'BD-20160112123540:3')
        self.assertEqual(self.item[0].get('uri'), 'BD-20160112123540')
        self.assertEqual(self.item[0].get('priority'), 6)
        self.assertEqual(self.item[0].get('version'), '3')
        self.assertEqual(self.item[0].get('firstcreated').isoformat(), '2016-01-11T23:35:40+00:00')
        self.assertEqual(self.item[0].get('urgency'), 3)
        self.assertTrue(self.item[0].get('body_html').startswith('<p>The receivers of Dick Smith Holdings'))
        self.assertEqual(self.item[0].get('byline'), 'Jonathan Underhill')
        self.assertEqual(self.item[0].get('dateline').get('located').get('city'), 'Wellington')
        self.assertEqual(self.item[0].get('place')[0].get('name'), 'NZ')

    def test_can_parse(self):
        self.assertTrue(ScoopNewsMLTwoFeedParser().can_parse(self.xml.getroot()))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from .vocabularies import VocabulariesResource, VocabulariesService
from .commands import VocabulariesPopulateCommand, UpdateVocabulariesInItemsCommand # noqa
import superdesk


def init_app(app):
    endpoint_name = 'vocabularies'
    service = VocabulariesService(endpoint_name, backend=superdesk.get_backend())
    VocabulariesResource(endpoint_name, app=app, service=service)

    superdesk.register_default_user_preference('cvs:preferred_items', {
        'value': {},
        'category': 'cvs',
        'label': 'Prefered CV items',
    })



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2016 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import io
import logging
import pstats
import re
from superdesk import get_resource_service
import superdesk

from flask import current_app as app

from superdesk.profiling.resource import ProfilingResource
from superdesk.profiling.service import ProfilingService, profile


logger = logging.getLogger(__name__)


def init_app(app):
    if app.config.get('ENABLE_PROFILING'):
        endpoint_name = 'profiling'
        service = ProfilingService(endpoint_name, backend=superdesk.get_backend())
        ProfilingResource(endpoint_name, app=app, service=service)

        superdesk.privilege(name='profiling', label='Profiling Service',
                            description='User can read profiling data.')

        profile.enable()


class ProfileManager():
    def __init__(self, name):
        self.name = name

    def __enter__(self):
        if app.config.get('ENABLE_PROFILING'):
            profile.enable()

    def __exit__(self, exc_type, exc_value, traceback):
        if app.config.get('ENABLE_PROFILING'):
            profile.disable()
            dump_stats(profile, self.name)


def dump_stats(profile, name):
    """Dump the profiling stats

    :param cProfile.Profile profile: the profile object
    :param string name: the name that identifies the profile
    """
    profile_output = io.StringIO()
    ps = pstats.Stats(profile, stream=profile_output).sort_stats('cumulative')
    ps.print_stats()
    lines = profile_output.getvalue().splitlines(True)
    profile_lines = []
    for line in lines:
        processed_line = _process_line(line)
        if processed_line:
            profile_lines.append(processed_line)
    profile_rec = get_resource_service('profiling').find_one(req=None, _id=name)
    if profile_rec:
        get_resource_service('profiling').patch(name, {'profiling_data': profile_lines})
    else:
        get_resource_service('profiling').post([{'name': name, 'profiling_data': profile_lines}])


def _process_line(line):
    """
    Reads from a text line the profiling fields into a dictionary.
    """
    LINE_FIELDS = {
        1: 'ncalls',
        2: 'tottime',
        3: 'percall',
        4: 'cumtime',
        5: 'percall_cumtime',
        6: 'filename_lineno',
        7: 'func_name'
    }
    LINE_REGEX = r'\s*([\d\/]+)\s+([\d.]+)\s+([\d.]+)\s+([\d.]+)\s+([\d.]+)\s+([^\(]+)\(([^\)]+)\)$'

    match = re.search(LINE_REGEX, line)
    if match:
        return {LINE_FIELDS[index]: match.group(index) for index in range(1, 8)}



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : edouard
# Creation: 2016-06-10 11:10

from superdesk.commands.data_updates import DataUpdate


class DataUpdate(DataUpdate):
    """

    Data update for Pull Request #418

    SD-3487 As a user I want to be able to use a template in several desks
    https://github.com/superdesk/superdesk-core/pull/418


    Changes:

        1. `template_desks` added and is a list of desks, initialized with desk `template_desk`
        2. `template_stage` renamed by `schedule_stage`
        3. `template_desk` renamed by `schedule_desk`

    """

    resource = 'content_templates'

    def forwards(self, mongodb_collection, mongodb_database):
        # new `template_desks` field must contain a list of desk id
        for template in mongodb_collection.find({}):
            if template.get('template_desk'):
                print(mongodb_collection.update({'_id': template['_id']}, {
                    '$set': {
                        'template_desks': [template.get('template_desk')]
                    }
                }))
        # renames fields:
        #   - template_desk -> schedule_desk
        #   - template_stage -> schedule_stage
        print(mongodb_collection.update({}, {
            '$rename': {
                'template_desk': 'schedule_desk',
                'template_stage': 'schedule_stage'
            },
        }, upsert=False, multi=True))

    def backwards(self, mongodb_collection, mongodb_database):
        print(mongodb_collection.update({}, {
            '$rename': {
                'schedule_desk': 'template_desk',
                'schedule_stage': 'template_stage'
            },
        }, upsert=False, multi=True))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk import get_backend
from superdesk.metadata.item import metadata_schema


def init_app(app):
    endpoint_name = 'spikes'
    service = BaseService(endpoint_name, backend=get_backend())
    SpikesResource(endpoint_name, app=app, service=service)


class SpikesResource(Resource):
    schema = metadata_schema
    datasource = {
        'source': 'archive',
        'search_backend': 'elastic',
        'default_sort': [('expiry', -1)],
        'elastic_filter': {'term': {'state': 'spiked'}}
    }
    resource_methods = ['GET']



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015, 2016 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import cProfile
import logging
import superdesk
from flask import current_app as app

from superdesk import get_resource_service
from superdesk.celery_task_utils import get_lock_id
from superdesk.lock import lock, unlock
from superdesk.metadata.item import ITEM_STATE, CONTENT_STATE, PUBLISH_SCHEDULE, SCHEDULE_SETTINGS
from apps.archive.common import ITEM_OPERATION, ARCHIVE, insert_into_versions
from apps.legal_archive.commands import import_into_legal_archive
from apps.publish.enqueue.enqueue_corrected import EnqueueCorrectedService
from apps.publish.enqueue.enqueue_killed import EnqueueKilledService
from apps.publish.enqueue.enqueue_published import EnqueuePublishedService
from apps.publish.published_item import PUBLISH_STATE, QUEUE_STATE, PUBLISHED, ERROR_MESSAGE
from bson.objectid import ObjectId
from eve.utils import config, ParsedRequest
from eve.versioning import resolve_document_version
from superdesk.celery_app import celery
from superdesk.utc import utcnow
from superdesk.profiling import ProfileManager
from apps.content import push_content_notification
from superdesk.errors import ConnectionTimeout


logger = logging.getLogger(__name__)

profile = cProfile.Profile()

UPDATE_SCHEDULE_DEFAULT = {'seconds': 10}

ITEM_PUBLISH = 'publish'
ITEM_CORRECT = 'correct'
ITEM_KILL = 'kill'
ITEM_TAKEDOWN = 'takedown'

enqueue_services = {
    ITEM_PUBLISH: EnqueuePublishedService(),
    ITEM_CORRECT: EnqueueCorrectedService(),
    ITEM_KILL: EnqueueKilledService(),
    ITEM_TAKEDOWN: EnqueueKilledService(),
}


def get_enqueue_service(operation):
    return enqueue_services[operation]


class EnqueueContent(superdesk.Command):
    """Runs deliveries"""

    def run(self):
        """Fetches items from publish queue as per the configuration, calls the transmit function.
        """
        lock_name = get_lock_id('publish', 'enqueue_published')
        if not lock(lock_name, expire=310):
            logger.info('Enqueue Task: {} is already running.'.format(lock_name))
            return

        try:
            items = get_published_items()

            if len(items) > 0:
                enqueue_items(items)
        finally:
            unlock(lock_name)


def enqueue_item(published_item):
    """
    Creates the corresponding entries in the publish queue for the given item
    """
    published_item_id = ObjectId(published_item[config.ID_FIELD])
    published_service = get_resource_service(PUBLISHED)
    archive_service = get_resource_service(ARCHIVE)
    published_update = {QUEUE_STATE: PUBLISH_STATE.IN_PROGRESS, 'last_queue_event': utcnow()}
    try:
        logger.info('Queueing item with id: {} and item_id: {}'.format(published_item_id, published_item['item_id']))

        published_item = published_service.find_one(req=None, _id=published_item_id)
        if published_item.get(QUEUE_STATE) != PUBLISH_STATE.PENDING:
            logger.info('Queue State is not pending for published item {}. It is in {}'.
                        format(published_item_id, published_item.get(QUEUE_STATE)))
            return

        if published_item.get(ITEM_STATE) == CONTENT_STATE.SCHEDULED:
            # if scheduled then change the state to published
            # change the `version` and `versioncreated` for the item
            # in archive collection and published collection.
            versioncreated = utcnow()
            item_updates = {'versioncreated': versioncreated, ITEM_STATE: CONTENT_STATE.PUBLISHED}
            resolve_document_version(document=item_updates, resource=ARCHIVE,
                                     method='PATCH',
                                     latest_doc={config.VERSION: published_item[config.VERSION]})

            # update the archive collection
            archive_item = archive_service.find_one(req=None, _id=published_item['item_id'])
            archive_service.system_update(published_item['item_id'], item_updates, archive_item)
            # insert into version.
            insert_into_versions(published_item['item_id'], doc=None)
            # update archive history
            app.on_archive_item_updated(item_updates, archive_item, ITEM_PUBLISH)
            # import to legal archive
            import_into_legal_archive.apply_async(countdown=3, kwargs={'item_id': published_item['item_id']})
            logger.info('Modified the version of scheduled item: {}'.format(published_item_id))

            logger.info('Publishing scheduled item_id: {}'.format(published_item_id))
            # update the published collection
            published_update.update(item_updates)
            published_item.update({'versioncreated': versioncreated,
                                   ITEM_STATE: CONTENT_STATE.PUBLISHED,
                                   config.VERSION: item_updates[config.VERSION]})
            # send a notification to the clients
            push_content_notification(
                [{'_id': str(published_item['item_id']), 'task': published_item.get('task', None)}])

        published_service.patch(published_item_id, published_update)
        # queue the item for publishing
        queued = get_enqueue_service(published_item[ITEM_OPERATION]).enqueue_item(published_item)

        # if the item is queued in the publish_queue then the state is "queued"
        # else the queue state is "queued_not_transmitted"
        queue_state = PUBLISH_STATE.QUEUED if queued else PUBLISH_STATE.QUEUED_NOT_TRANSMITTED
        published_service.patch(published_item_id, {QUEUE_STATE: queue_state})
        logger.info('Queued item with id: {} and item_id: {}'.format(published_item_id, published_item['item_id']))
    except KeyError as key_error:
        error_updates = {QUEUE_STATE: PUBLISH_STATE.ERROR, ERROR_MESSAGE: str(key_error)}
        published_service.patch(published_item_id, error_updates)
        logger.exception('No enqueue service found for operation %s', published_item[ITEM_OPERATION])
    except ConnectionTimeout as error:  # recoverable, set state to pending and retry next time
        error_updates = {QUEUE_STATE: PUBLISH_STATE.PENDING, ERROR_MESSAGE: str(error)}
        published_service.patch(published_item_id, error_updates)
        raise
    except Exception as error:
        error_updates = {QUEUE_STATE: PUBLISH_STATE.ERROR, ERROR_MESSAGE: str(error)}
        published_service.patch(published_item_id, error_updates)
        raise


def get_published_items():
    """
    Get all items with queue state: "pending" that are not scheduled or scheduled time has lapsed.
    """
    query = {
        QUEUE_STATE: PUBLISH_STATE.PENDING,
        '$or': [
            {
                ITEM_STATE: {'$ne': CONTENT_STATE.SCHEDULED}
            },
            {
                ITEM_STATE: CONTENT_STATE.SCHEDULED,
                '{}.utc_{}'.format(SCHEDULE_SETTINGS, PUBLISH_SCHEDULE): {'$lte': utcnow()}
            }
        ]
    }
    request = ParsedRequest()
    request.sort = 'publish_sequence_no'
    request.max_results = 200
    return list(get_resource_service(PUBLISHED).get_from_mongo(req=request, lookup=query))


def enqueue_items(published_items):
    """Creates the corresponding entries in the publish queue for each item

    :param list published_items: the list of items marked for publishing
    """
    failed_items = {}

    for queue_item in published_items:
        try:
            enqueue_item(queue_item)
        except Exception:
            logger.exception('Failed to queue item {}'.format(queue_item.get('_id')))
            failed_items[str(queue_item.get('_id'))] = queue_item

    if len(failed_items) > 0:
        logger.error('Failed to publish the following items: {}'.format(failed_items.keys()))


superdesk.command('publish:enqueue', EnqueueContent())


@celery.task(soft_time_limit=300)
def enqueue_published():
    """Pick new items from ``published`` collection and enqueue it."""
    with ProfileManager('publish:enqueue'):
        EnqueueContent().run()



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : mugur
# Creation: 2018-04-25 01:07

from eve.utils import config
from superdesk.commands.data_updates import DataUpdate


class DataUpdate(DataUpdate):

    resource = 'vocabularies'
    update_fields = ['name', 'qcode']

    def forwards(self, mongodb_collection, mongodb_database):
        for vocabulary in mongodb_collection.find({}):
            if 'schema' in vocabulary:
                schema = vocabulary['schema']
                for field in self.update_fields:
                    if field in vocabulary['schema'] and type(vocabulary['schema']) == dict:
                        schema[field]['required'] = True
                mongodb_collection.update({'_id': vocabulary.get(config.ID_FIELD)},
                                          {'$set': {'schema': schema}})

    def backwards(self, mongodb_collection, mongodb_database):
        pass



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import logging

from eve.utils import config
from flask import json

from apps.archive.archive import SOURCE as ARCHIVE
from apps.archive.common import generate_unique_id_and_name, ITEM_OPERATION
from apps.archive.common import insert_into_versions, remove_unwanted, set_original_creator
from apps.duplication.archive_fetch import ITEM_FETCH
from apps.tasks import send_to
from superdesk import get_resource_service
import superdesk
from superdesk.errors import SuperdeskApiError, ProviderError
from superdesk.metadata.item import GUID_TAG, FAMILY_ID, INGEST_ID, ITEM_STATE, CONTENT_STATE
from superdesk.metadata.utils import generate_guid
from superdesk.resource import Resource
from superdesk.utc import utcnow

logger = logging.getLogger(__name__)


class ProviderNotFoundError(SuperdeskApiError):
    pass


class SearchIngestResource(superdesk.Resource):
    resource_methods = ['GET', 'POST']
    schema = {
        'guid': {'type': 'string', 'required': True},
        'desk': Resource.rel('desks', False, nullable=True),
        'stage': Resource.rel('stages', False, nullable=True)
    }


class SearchIngestService(superdesk.Service):
    def __init__(self, datasource=None, backend=None, source=None):
        super().__init__(datasource, backend)
        self.source = source

    def get_provider(self):
        provider = get_resource_service('search_providers').find_one(source=self.source, req=None)
        if provider and 'config' in provider and 'username' in provider['config']:
            self.backend.set_credentials(provider['config']['username'], provider['config'].get('password', ''))
        return provider

    def fetch(self, guid):
        return self.backend.find_one_raw(guid, guid)

    def create(self, docs, **kwargs):
        new_guids = []
        provider = self.get_provider()
        for doc in docs:
            if not doc.get('desk'):
                # if no desk is selected then it is bad request
                raise SuperdeskApiError.badRequestError("Destination desk cannot be empty.")
            try:
                archived_doc = self.fetch(doc['guid'])
            except FileNotFoundError as ex:
                raise ProviderError.externalProviderError(ex, provider)

            dest_doc = dict(archived_doc)
            new_id = generate_guid(type=GUID_TAG)
            new_guids.append(new_id)
            dest_doc['_id'] = new_id
            generate_unique_id_and_name(dest_doc)

            if provider:
                dest_doc['ingest_provider'] = str(provider[superdesk.config.ID_FIELD])

            dest_doc[config.VERSION] = 1
            send_to(doc=dest_doc, update=None, desk_id=doc.get('desk'), stage_id=doc.get('stage'))
            dest_doc[ITEM_STATE] = doc.get(ITEM_STATE, CONTENT_STATE.FETCHED)
            dest_doc[INGEST_ID] = archived_doc['_id']
            dest_doc[FAMILY_ID] = archived_doc['_id']
            dest_doc[ITEM_OPERATION] = ITEM_FETCH
            remove_unwanted(dest_doc)
            set_original_creator(dest_doc)

            superdesk.get_resource_service(ARCHIVE).post([dest_doc])
            insert_into_versions(dest_doc.get('_id'))

        if new_guids:
            get_resource_service('search_providers').system_update(provider.get(config.ID_FIELD),
                                                                   {'last_item_update': utcnow()}, provider)

        return new_guids

    def get(self, req, lookup):
        provider = self.get_provider()
        if provider:
            query = self._get_query(req)
            results = self.backend.find(self.source, query, None)
            for doc in results.docs:
                doc['ingest_provider'] = str(provider[superdesk.config.ID_FIELD])
            return results
        else:
            raise ProviderNotFoundError('provider not found source=%s' % (self.source, ))

    def fetch_rendition(self, rendition):
        """Get file stream for given rendition specs.

        Rendition should be from item that was fetched via this service get method.
        It can use api authentication if needed to fetch this binary.

        :param rendition: rendition dict
        """
        self.get_provider()
        return self.backend.fetch_file(rendition.get('href'))

    def _get_query(self, req):
        args = getattr(req, 'args', {})
        query = json.loads(args.get('source')) if args.get('source') else {'query': {'filtered': {}}}
        return query



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2016 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
#

"""Schema utils."""

import superdesk

from flask import current_app as app
from superdesk.lock import lock, unlock
from superdesk.commands.rebuild_elastic_index import RebuildElasticIndex


VERSION_ID = 'schema_version'


def _get_version_db():
    """Get db used for storing version information."""
    return app.data.mongo.pymongo().db['superdesk']


def get_schema_version():
    """Read app schema version from db."""
    db = _get_version_db()
    version = db.find_one({'_id': VERSION_ID})
    return version.get('version') if version else 0


def set_schema_version(version):
    """Store app schema version to db.

    :param version
    """
    db = _get_version_db()
    db.update_one({'_id': VERSION_ID}, {'$set': {'version': version}}, upsert=True)


def update_schema():
    """Run rebuild elastic index command to update schema."""
    RebuildElasticIndex().run()


class SchemaMigrateCommand(superdesk.Command):
    """Migrate elastic schema if needed, should be triggered on every deploy.

    It compares version set in code (latest) to one stored in db and only updates
    schema if those are different.

    Current version is read from settings and fallbacks to superdesk.SCHEMA_VERSION,
    so that you can avoid migration via settings file if needed.
    """

    def run(self):
        lock_name = 'schema:migrate'

        if not lock(lock_name, expire=1800):
            return

        try:
            app_schema_version = get_schema_version()
            superdesk_schema_version = app.config.get('SCHEMA_VERSION', superdesk.SCHEMA_VERSION)
            if app_schema_version < superdesk.SCHEMA_VERSION:
                print('Updating schema from version {} to {}.'.format(
                    app_schema_version, superdesk_schema_version
                ))
                update_schema()
                set_schema_version(superdesk_schema_version)
            else:
                print('App already at version ({}).'.format(app_schema_version))
        finally:
            unlock(lock_name)


superdesk.command('schema:migrate', SchemaMigrateCommand())



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : Jérôme
# Creation: 2017-09-08 14:02

from superdesk.commands.data_updates import DataUpdate


class DataUpdate(DataUpdate):

    resource = 'users'

    def forwards(self, mongodb_collection, mongodb_database):
        # we want all existing users to be authors by default
        print(mongodb_collection.update_many({'is_author': {'$exists': False}},
                                             {'$set': {
                                                 'is_author': True
                                             }}))

    def backwards(self, mongodb_collection, mongodb_database):
        # author was not existing before the update, so we remove the value
        print(mongodb_collection.update_many({},
                                             {'$unset': {
                                                 'is_author': ''
                                             }}))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource


class FormattersResource(Resource):
    """Formatters schema"""

    endpoint_name = 'formatters'
    resource_methods = ['GET', 'POST']
    item_methods = []
    resource_title = endpoint_name
    schema = {
        'article_id': {
            'type': 'string',
            'required': True
        },
        'formatter_name': {
            'type': 'string',
            'required': True
        }
    }
    privileges = {'POST': 'archive'}



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : sdesk
# Creation: 2016-07-22 11:16

from superdesk.commands.data_updates import DataUpdate
from superdesk import get_resource_service
from eve.utils import config


class DataUpdate(DataUpdate):
    """Updates the user collection with invisible stages.

    Refer to https://dev.sourcefabric.org/browse/SD-5077 for more information
    """

    resource = 'users'

    def forwards(self, mongodb_collection, mongodb_database):
        for user in mongodb_collection.find({}):
            stages = get_resource_service(self.resource).get_invisible_stages_ids(user.get(config.ID_FIELD))
            print(mongodb_collection.update({'_id': user.get(config.ID_FIELD)},
                                            {'$set': {
                                                'invisible_stages': stages
                                            }}))

    def backwards(self, mongodb_collection, mongodb_database):
        print(mongodb_collection.update({},
                                        {'$unset': {'invisible_stages': []}},
                                        upsert=False, multi=True))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from superdesk.io.feed_parsers.ritzau import RitzauFeedParser
from superdesk.tests import TestCase
from superdesk.etree import etree
import os


class BaseRitzauTestCase(TestCase):

    vocab = [{'_id': 'genre', 'items': [{'name': 'Current'}]}]

    def setUp(self):
        with self.app.app_context():
            self.app.data.insert('vocabularies', self.vocab)
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, '../fixtures', self.filename))
        provider = {'name': 'Test'}
        with open(fixture, 'rb') as f:
            self.root_elt = etree.fromstring(f.read())
            self.item = RitzauFeedParser().parse(self.root_elt, provider)


class RitzauTestCase(BaseRitzauTestCase):
    filename = 'ritzau_news.xml'

    def test_can_parse(self):
        self.assertTrue(RitzauFeedParser().can_parse(self.root_elt))

    def test_content(self):
        item = self.item
        self.assertEqual(item['version'], 1)
        self.assertEqual(item['byline'], '/ritzau/')
        self.assertEqual(item['headline'], 'Hollandske forskere har "i årevis" lavet forsøg på både mennesker og '
                                           'dyr, hvor de har testet effekte')
        self.assertEqual(item['body_html'], '<p>Hollandske forskere har "i årevis" lavet forsøg på både '
                                            'mennesker og dyr, hvor de har testet effekten af at indånde '
                                            'udstødningsgas fra dieselbiler.</p><p>Det fortæller forskerne '
                                            'selv til nyhedsbureauet AFP.</p><p>Historien kommer frem på et '
                                            'tidspunkt, hvor tyske bilfabrikanter er udsat for massiv '
                                            'kritik, fordi de har finansieret lignende forsøg.</p>')
        self.assertEqual(item['guid'], '9a6955fc-11da-46b6-9903-439ebb288f2d')
        self.assertEqual(item['firstcreated'].isoformat(), '2018-01-30T17:32:18.397000+01:00')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import superdesk
import elasticsearch

from flask import current_app
from eve_elastic import get_es, get_indices, reindex
from superdesk.utils import get_random_string


class RebuildElasticIndex(superdesk.Command):
    """Rebuild the elastic indexes from existing data.

    It creates new index with same alias as the configured index,
    puts the new mapping and removes the old index.
    """

    option_list = [
        superdesk.Option('--index', '-i', dest='index_name')
    ]

    def run(self, index_name=None):
        # if no index name is passed then use the configured one
        indexes = list(current_app.data.elastic._get_indexes().keys())
        if index_name and index_name in indexes:
            indexes = [index_name]
        elif index_name:
            raise Exception("Index {} is not configured".format(index_name))
        for index_name in indexes:
            try:
                print('Starting index rebuilding for index: {}'.format(index_name))
                es = get_es(superdesk.app.config['ELASTICSEARCH_URL'])
                clone_name = index_name + '-' + get_random_string()
                print('Creating index: ', clone_name)
                superdesk.app.data.elastic.create_index(clone_name, superdesk.app.config['ELASTICSEARCH_SETTINGS'])
                real_name = superdesk.app.data.elastic.get_index_by_alias(clone_name)
                print('Putting mapping for index: ', clone_name)
                superdesk.app.data.elastic.put_mapping(superdesk.app, clone_name)
                print('Starting index rebuilding.')
                reindex(es, index_name, clone_name)
                print('Finished index rebuilding.')
                print('Deleting index: ', index_name)
                get_indices(es).delete(index_name)
                print('Creating alias: ', index_name)
                get_indices(es).put_alias(index=real_name, name=index_name)
                print('Alias created.')
                print('Deleting clone name alias')
                get_indices(es).delete_alias(name=clone_name, index=real_name)
                print('Deleted clone name alias')
            except elasticsearch.exceptions.NotFoundError as nfe:
                print(nfe)
            print('Index {0} rebuilt successfully.'.format(index_name))


superdesk.command('app:rebuild_elastic_index', RebuildElasticIndex())



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : tomas
# Creation: 2018-03-21 09:28

import superdesk
import subprocess
import json
from superdesk.commands.data_updates import DataUpdate
from os.path import realpath, join, dirname

node_script_path = join(dirname(realpath(superdesk.__file__)), "data_updates", "00007_20180321-092824_archive.dist.js")


def get_updated_editor_state(editor_state):
    try:
        with subprocess.Popen(['node', node_script_path], stdin=subprocess.PIPE, stdout=subprocess.PIPE) as p:
            output, err = p.communicate(bytes(json.dumps(editor_state), 'utf-8'))
            return json.loads(output.decode('utf-8'))
    except Exception:
        return editor_state


class DataUpdate(DataUpdate):

    resource = 'archive'  # will use multiple resources, keeping this here so validation passes

    def forwards(self, mongodb_collection, mongodb_database):
        for resource in ['archive', 'archive_autosave', 'published']:

            collection = mongodb_database[resource]

            for item in collection.find({'editor_state': {'$exists': True}}):
                print(collection.update({'_id': item['_id']}, {
                    '$set': {
                        'editor_state': get_updated_editor_state(item['editor_state'])
                    }
                }))

    def backwards(self, mongodb_collection, mongodb_database):
        pass



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import glob
import os
import shutil
import tempfile
import unittest
from unittest import mock
import datetime
import pytz

from superdesk.io.feeding_services import ftp
from superdesk.utc import utcnow

PREFIX = 'test_superdesk_'
PROVIDER = {
    "_id": "test_provider",
    "config": {
        "passive": True,
        "username": "user",
        "password": "password",
        "host": "example.net",
        "dest_path": "/tmp",
        "path": "",
        "move": True,
        "move_path": "dest_move",
        "move_path_error": "error",
        "field_aliases": []
    },
    "last_updated": datetime.datetime(2017, 5, 16, 16, 47, 39, tzinfo=pytz.utc),
    "last_item_update": datetime.datetime(2017, 5, 16, 16, 47, 39, tzinfo=pytz.utc)
}


def ftp_file(filename):
    facts = mock.Mock()
    facts.get.return_value = 'file'
    facts.__getitem__ = mock.Mock(return_value="20170517164739")
    return [filename, facts]


class FakeFTP(mock.MagicMock):

    files = [ftp_file('filename.xml')]

    def mlsd(self, path=""):
        return iter(self.files)

    def cwd(self, path):
        pass


class FakeFeedParser(mock.MagicMock):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        del self.ALLOWED_EXT


class FailingFakeFeedParser(FakeFeedParser):

    def parse(self, *args, **kwargs):
        raise Exception('Test exception')


class FTPTestCase(unittest.TestCase):

    def test_it_can_connect(self):
        service = ftp.FTPFeedingService()

        if 'FTP_URL' not in os.environ:
            return

        config = service.config_from_url(os.environ['FTP_URL'])
        self.assertEqual('test', config['path'])
        self.assertEqual('localhost', config['host'])

        config['dest_path'] = tempfile.mkdtemp(prefix=PREFIX)
        provider = {'config': config}

        items = service._update(provider, {})
        self.assertEqual(266, len(items))

        provider['last_updated'] = utcnow()
        self.assertEqual(0, len(service._update(provider, {})))

        self.assertTrue(os.path.isdir(provider['config']['dest_path']))
        self.assertEqual(266, len(os.listdir(provider['config']['dest_path'])))

    def tearDown(self):
        for folder in glob.glob('/tmp/%s*' % (PREFIX)):
            shutil.rmtree(folder)

    @mock.patch.object(ftp, 'ftp_connect', new_callable=FakeFTP)
    @mock.patch.object(ftp.FTPFeedingService, 'get_feed_parser', FakeFeedParser())
    @mock.patch('builtins.open', mock.mock_open())
    def test_move_ingested(self, ftp_connect):
        """Check that ingested file is moved if "move" is set

        feature requested in SDESK-468
        """
        provider = PROVIDER.copy()
        service = ftp.FTPFeedingService()
        service._update(provider, {})
        mock_ftp = ftp_connect.return_value.__enter__.return_value
        mock_ftp.rename.assert_called_once_with('filename.xml', 'dest_move/filename.xml')

    @mock.patch.object(ftp, 'ftp_connect', new_callable=FakeFTP)
    @mock.patch.object(ftp.FTPFeedingService, 'get_feed_parser', FakeFeedParser())
    @mock.patch('builtins.open', mock.mock_open())
    def test_move_ingested_default(self, ftp_connect):
        """Check that ingested file is moved to default path if "move" is empty string

        feature requested in SDESK-1452
        """
        provider = PROVIDER.copy()
        provider['config']['move_path'] = ""
        service = ftp.FTPFeedingService()
        service._update(provider, {})
        mock_ftp = ftp_connect.return_value.__enter__.return_value
        dest_path = os.path.join(ftp.DEFAULT_SUCCESS_PATH, "filename.xml")
        mock_ftp.rename.assert_called_once_with('filename.xml', dest_path)

    @mock.patch.object(ftp, 'ftp_connect', new_callable=FakeFTP)
    @mock.patch.object(ftp.FTPFeedingService, 'get_feed_parser', FakeFeedParser())
    @mock.patch('builtins.open', mock.mock_open())
    def test_move_ingested_no_move(self, ftp_connect):
        """Check that ingested file is not moved if "move" is not set

        feature requested in SDESK-468
        """
        provider = PROVIDER.copy()
        provider['config']['move'] = False
        service = ftp.FTPFeedingService()
        service._update(provider, {})
        mock_ftp = ftp_connect.return_value.__enter__.return_value
        mock_ftp.rename.assert_not_called()

    @mock.patch.object(ftp, 'ftp_connect', new_callable=FakeFTP)
    @mock.patch.object(ftp.FTPFeedingService, 'get_feed_parser', FailingFakeFeedParser())
    @mock.patch('builtins.open', mock.mock_open())
    def test_move_error(self, ftp_connect):
        """Check that error on ingestion moves item if "move_path_error" is set

        feature requested in SDESK-1452
        """
        provider = PROVIDER.copy()
        service = ftp.FTPFeedingService()
        service._update(provider, {})
        mock_ftp = ftp_connect.return_value.__enter__.return_value
        mock_ftp.rename.assert_called_once_with('filename.xml', 'error/filename.xml')

    @mock.patch.object(ftp, 'ftp_connect', new_callable=FakeFTP)
    @mock.patch.object(ftp.FTPFeedingService, 'get_feed_parser', FailingFakeFeedParser())
    @mock.patch('builtins.open', mock.mock_open())
    def test_move_error_default(self, ftp_connect):
        """Check that error on ingestion use default path if "move_path_error" is empty string

        feature requested in SDESK-1452
        """
        provider = PROVIDER.copy()
        provider['config']['move_path_error'] = ""
        service = ftp.FTPFeedingService()
        service._update(provider, {})
        mock_ftp = ftp_connect.return_value.__enter__.return_value
        dest_path = os.path.join(ftp.DEFAULT_FAILURE_PATH, "filename.xml")
        mock_ftp.rename.assert_called_once_with('filename.xml', dest_path)

    def test_allowed_suffix_json(self):
        """Check that json files are allowed for ingestion."""
        service = ftp.FTPFeedingService()
        allowed = ftp.FTPFeedingService.ALLOWED_EXT_DEFAULT
        self.assertFalse(service._is_allowed('foo.jpg', allowed))
        self.assertTrue(service._is_allowed('foo.xml', allowed))
        self.assertTrue(service._is_allowed('foo.json', allowed))
        self.assertTrue(service._is_allowed('foo.JSON', allowed))
        self.assertFalse(service._is_allowed('foojson', allowed))
        self.assertFalse(service._is_allowed('foo.json.tar.gz', allowed))
